@@ -39,6 +39,7 @@
 #include <net-snmp/agent/agent_index.h>
 
 #include "snmpd.h"
+#include "agent_global_vars.h"
 #include "mibgroup/struct.h"
 #include <net-snmp/agent/table.h>
 #include <net-snmp/agent/table_iterator.h>
@@ -65,8 +66,6 @@ struct snmp_index {
     struct snmp_index *next_idx;
 }              *snmp_index_head = NULL;
 
-extern netsnmp_session *main_session;
-
 /*
  * The caller is responsible for free()ing the memory returned by
  * this function.  
