@@ -441,6 +441,7 @@ agentx_realloc_build_varbind(u_char ** buf, size_t * buf_len,
                                   vp->name, vp->name_length,
                                   network_order)) {
         DEBUGINDENTLESS();
+        DEBUGINDENTLESS();
         return 0;
     }
     DEBUGINDENTLESS();
@@ -452,6 +453,7 @@ agentx_realloc_build_varbind(u_char ** buf, size_t * buf_len,
     case ASN_COUNTER:
     case ASN_GAUGE:
     case ASN_TIMETICKS:
+    case ASN_UINTEGER:
         if (!agentx_realloc_build_int(buf, buf_len, out_len, allow_realloc,
                                       *(vp->val.integer), network_order)) {
             DEBUGINDENTLESS();
@@ -539,6 +541,7 @@ agentx_realloc_build_varbind(u_char ** buf, size_t * buf_len,
                 DEBUGINDENTLESS();
                 return 0;
             }
+            DEBUGINDENTLESS();
         } else {
             DEBUGDUMPHEADER("send", "Build Counter64 (low, high)");
             if (!agentx_realloc_build_int
@@ -553,6 +556,7 @@ agentx_realloc_build_varbind(u_char ** buf, size_t * buf_len,
                 DEBUGINDENTLESS();
                 return 0;
             }
+            DEBUGINDENTLESS();
         }
         break;
 
@@ -668,7 +672,7 @@ _agentx_realloc_build(u_char ** buf, size_t * buf_len, size_t * out_len,
                       int allow_realloc,
                       netsnmp_session * session, netsnmp_pdu *pdu)
 {
-    size_t          ilen = *out_len, prefix_offset = 0;
+    size_t          ilen = *out_len;
     netsnmp_variable_list *vp;
     int             inc, i = 0;
     const int       network_order =
@@ -821,7 +825,6 @@ _agentx_realloc_build(u_char ** buf, size_t * buf_len, size_t * out_len,
         DEBUGINDENTLESS();
 
         vp = pdu->variables;
-        prefix_offset = *out_len + 1;
         DEBUGDUMPHEADER("send", "(Un)Register Prefix");
         if (!agentx_realloc_build_oid
             (buf, buf_len, out_len, allow_realloc, 0, vp->name,
@@ -869,9 +872,7 @@ _agentx_realloc_build(u_char ** buf, size_t * buf_len, size_t * out_len,
         }
         DEBUGINDENTLESS();
 
-        /*
-         * Fallthrough  
-         */
+        /* FALL THROUGH */
 
     case AGENTX_MSG_GET:
     case AGENTX_MSG_GETNEXT:
@@ -926,9 +927,7 @@ _agentx_realloc_build(u_char ** buf, size_t * buf_len, size_t * out_len,
         DEBUGMSG(("dumpv_send", "  errindex:\t%ld\n", pdu->errindex));
         DEBUGINDENTLESS();
 
-        /*
-         * Fallthrough  
-         */
+        /* FALL THROUGH */
 
     case AGENTX_MSG_INDEX_ALLOCATE:
     case AGENTX_MSG_INDEX_DEALLOCATE:
@@ -1161,7 +1160,9 @@ agentx_parse_oid(u_char * data, size_t * length, int *inc,
      */
     tmp_oid_len = (prefix ? n_subid + 5 : n_subid);
     if (*oid_len < tmp_oid_len) {
-        DEBUGMSGTL(("agentx", "Oversized Object ID\n"));
+        DEBUGMSGTL(("agentx", "Oversized Object ID (buf=%" NETSNMP_PRIz "d"
+		    " pdu=%d)\n", *oid_len, tmp_oid_len));
+        DEBUGINDENTLESS();
         return NULL;
     }
 
@@ -1172,6 +1173,7 @@ agentx_parse_oid(u_char * data, size_t * length, int *inc,
 #endif
     if (*length < 4 * n_subid) {
         DEBUGMSGTL(("agentx", "Incomplete Object ID\n"));
+        DEBUGINDENTLESS();
         return NULL;
     }
 
@@ -1240,7 +1242,7 @@ agentx_parse_string(u_char * data, size_t * length,
                     (int)*length));
         return NULL;
     }
-    if (len > *str_len) {
+    if (len > (*str_len - 1)) {
         DEBUGMSGTL(("agentx", "String too long (too long)\n"));
         return NULL;
     }
@@ -1252,6 +1254,12 @@ agentx_parse_string(u_char * data, size_t * length,
     len >>= 2;
     len <<= 2;
 
+    if (*length < len + 4) {
+        DEBUGMSGTL(("agentx", "Packet too short for string padding (still too short: %d)\n",
+                    (int)*length));
+        return NULL;
+    }
+
     *length -= (len + 4);
     DEBUGDUMPSETUP("recv", data, (len + 4));
     DEBUGIF("dumpv_recv") {
@@ -1276,6 +1284,7 @@ agentx_parse_opaque(u_char * data, size_t * length, int *type,
                     u_char * opaque_buf, size_t * opaque_len,
                     u_int network_byte_order)
 {
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
     union {
         float           floatVal;
         double          doubleVal;
@@ -1284,17 +1293,18 @@ agentx_parse_opaque(u_char * data, size_t * length, int *type,
     } fu;
     int             tmp;
     u_char         *buf;
-    u_char         *cp;
+#endif
+    u_char         *const cp =
+        agentx_parse_string(data, length,
+                            opaque_buf, opaque_len, network_byte_order);
 
-    cp = agentx_parse_string(data, length,
-                             opaque_buf, opaque_len, network_byte_order);
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
     if (cp == NULL)
         return NULL;
 
     buf = opaque_buf;
 
-#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
-    if ((buf[0] != ASN_OPAQUE_TAG1) || (*opaque_len <= 3))
+    if ((*opaque_len <= 3) || (buf[0] != ASN_OPAQUE_TAG1))
         return cp;              /* Unrecognised opaque type */
 
     switch (buf[1]) {
@@ -1348,6 +1358,8 @@ agentx_parse_varbind(u_char * data, size_t * length, int *type,
     u_int           int_val;
     struct counter64 tmp64;
 
+    if (*length < 4)
+        return NULL;
     DEBUGDUMPHEADER("recv", "VarBind:");
     DEBUGDUMPHEADER("recv", "Type");
     *type = agentx_parse_short(bufp, network_byte_order);
@@ -1367,6 +1379,11 @@ agentx_parse_varbind(u_char * data, size_t * length, int *type,
     case ASN_COUNTER:
     case ASN_GAUGE:
     case ASN_TIMETICKS:
+    case ASN_UINTEGER:
+        if (*length < 4) {
+            DEBUGINDENTLESS();
+            return NULL;
+        }
         int_val = agentx_parse_int(bufp, network_byte_order);
         memmove(data_buf, &int_val, 4);
         *data_len = 4;
@@ -1398,6 +1415,10 @@ agentx_parse_varbind(u_char * data, size_t * length, int *type,
         break;
 
     case ASN_COUNTER64:
+        if (*length < 8) {
+            DEBUGINDENTLESS();
+            return NULL;
+        }
         memset(&tmp64, 0, sizeof(tmp64));
 	if (network_byte_order) {
 	    tmp64.high = agentx_parse_int(bufp,   network_byte_order);
@@ -1424,6 +1445,7 @@ agentx_parse_varbind(u_char * data, size_t * length, int *type,
         break;
 
     default:
+        DEBUGMSG(("recv", "Can not parse type %x", *type));
         DEBUGINDENTLESS();
         return NULL;
     }
@@ -1526,6 +1548,7 @@ agentx_parse_header(netsnmp_pdu *pdu, u_char * data, size_t * length)
     DEBUGINDENTLESS();
     bufp += 4;
 
+    DEBUGINDENTLESS();
     *length -= 20;
     if (*length != payload) {   /* Short payload */
         return NULL;
@@ -1539,8 +1562,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
              size_t len)
 {
     register u_char *bufp = data;
-    u_char          buffer[SNMP_MAX_MSG_SIZE];
-    u_char         *prefix_ptr;
+    u_char          buffer[65536];
     oid             oid_buffer[MAX_OID_LEN], end_oid_buf[MAX_OID_LEN];
     size_t          buf_len = sizeof(buffer);
     size_t          oid_buf_len = MAX_OID_LEN;
@@ -1551,14 +1573,9 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
     int             type;       /* VarBind data type */
     size_t         *length = &len;
 
-    if (pdu == NULL) {
-        /*
-         * Dump the packet in a formatted style 
-         */
-        pdu = (netsnmp_pdu *) malloc(sizeof(netsnmp_pdu));
-        free(pdu);
+    if (pdu == NULL)
         return (0);
-    }
+ 
     if (!IS_AGENTX_VERSION(session->version))
         return SNMPERR_BAD_VERSION;
 
@@ -1621,10 +1638,15 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         buf_len = sizeof(buffer);
     }
 
+#define AGENTX_NEEDLEN( len ) \
+        if ( *length < len ) { \
+            DEBUGINDENTLESS(); \
+            return SNMPERR_ASN_PARSE_ERR; \
+        }
     DEBUGDUMPHEADER("recv", "PDU");
-    DEBUGINDENTMORE();
     switch (pdu->command) {
     case AGENTX_MSG_OPEN:
+        AGENTX_NEEDLEN(4);
         pdu->time = *bufp;      /* Timeout */
         bufp += 4;
         *length -= 4;
@@ -1660,6 +1682,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         break;
 
     case AGENTX_MSG_CLOSE:
+        AGENTX_NEEDLEN(4);
         pdu->errstat = *bufp;   /* Reason */
         bufp += 4;
         *length -= 4;
@@ -1668,6 +1691,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
 
     case AGENTX_MSG_UNREGISTER:
     case AGENTX_MSG_REGISTER:
+        AGENTX_NEEDLEN(4);
         DEBUGDUMPHEADER("recv", "Registration Header");
         if (pdu->command == AGENTX_MSG_REGISTER) {
             pdu->time = *bufp;  /* Timeout (Register only) */
@@ -1687,7 +1711,6 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         *length -= 4;
         DEBUGINDENTLESS();
 
-        prefix_ptr = bufp + 1;
         DEBUGDUMPHEADER("recv", "Registration OID");
         bufp = agentx_parse_oid(bufp, length, NULL,
                                 oid_buffer, &oid_buf_len,
@@ -1699,6 +1722,11 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         }
 
         if (pdu->range_subid) {
+            if (pdu->range_subid > oid_buf_len) {
+                DEBUGINDENTLESS();
+                return SNMPERR_ASN_PARSE_ERR;
+            }
+            AGENTX_NEEDLEN(4);
             range_bound = agentx_parse_int(bufp, pdu->flags &
                                            AGENTX_FLAGS_NETWORK_BYTE_ORDER);
             bufp += 4;
@@ -1722,6 +1750,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         break;
 
     case AGENTX_MSG_GETBULK:
+        AGENTX_NEEDLEN(4);
         DEBUGDUMPHEADER("recv", "Non-repeaters");
         pdu->non_repeaters = agentx_parse_short(bufp, pdu->flags &
                                                 AGENTX_FLAGS_NETWORK_BYTE_ORDER);
@@ -1732,9 +1761,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         DEBUGINDENTLESS();
         bufp += 4;
         *length -= 4;
-        /*
-         * Fallthrough - SearchRange handling is the same 
-         */
+        /* FALLTHROUGH */
 
     case AGENTX_MSG_GETNEXT:
     case AGENTX_MSG_GET:
@@ -1791,6 +1818,8 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
 
         pdu->flags |= UCD_MSG_FLAG_RESPONSE_PDU;
 
+        AGENTX_NEEDLEN(8);
+
         /*
          * sysUpTime 
          */
@@ -1806,9 +1835,7 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
                                pdu->flags & AGENTX_FLAGS_NETWORK_BYTE_ORDER);
         bufp += 4;
         *length -= 4;
-        /*
-         * Fallthrough - VarBind handling is the same 
-         */
+        /* FALL THROUGH */
 
     case AGENTX_MSG_INDEX_ALLOCATE:
     case AGENTX_MSG_INDEX_DEALLOCATE:
@@ -1888,17 +1915,15 @@ agentx_parse(netsnmp_session * session, netsnmp_pdu *pdu, u_char * data,
         break;
 
     default:
-        DEBUGINDENTLESS();
         DEBUGINDENTLESS();
         DEBUGMSGTL(("agentx", "Unrecognised PDU type: %d\n",
                     pdu->command));
         return SNMPERR_UNKNOWN_PDU;
     }
     DEBUGINDENTLESS();
-    DEBUGINDENTLESS();
-    DEBUGINDENTLESS();
     return SNMP_ERR_NOERROR;
 }
+#undef AGENTX_NEEDLEN
 
 
 
