@@ -154,7 +154,7 @@ struct rtnl_handle {
 
 struct ifstat_ent {
     struct ifstat_ent *next;
-    const char *name;
+    char *name;
     int ifindex;
     struct rtnl_link_stats stats;
 };
@@ -465,6 +465,7 @@ _dot3Stats_netlink_get_errorcntrs(dot3StatsTable_rowreq_ctx *rowreq_ctx, const c
             done = 1;
         }
         kern_db = ke->next;
+        free(ke->name);
         free(ke);
     }
 
@@ -740,6 +741,11 @@ interface_ioctl_dot3stats_get (dot3StatsTable_rowreq_ctx *rowreq_ctx, int fd, co
             rowreq_ctx->column_exists_flags |= COLUMN_DOT3STATSALIGNMENTERRORS_FLAG;
         }
 
+        if (DOT3STATSFCSERRORS(s)) {
+            data->dot3StatsFCSErrors = (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_DOT3STATSFCSERRORS_FLAG;
+        }
+
         if (DOT3STATSMULTIPLECOLLISIONFRAMES(s)) {
             data->dot3StatsMultipleCollisionFrames = (u_long)eth_stats->data[i];
             rowreq_ctx->column_exists_flags |= COLUMN_DOT3STATSMULTIPLECOLLISIONFRAMES_FLAG;
@@ -759,6 +765,10 @@ interface_ioctl_dot3stats_get (dot3StatsTable_rowreq_ctx *rowreq_ctx, int fd, co
             data->dot3StatsExcessiveCollisions = (u_long)eth_stats->data[i];
             rowreq_ctx->column_exists_flags |= COLUMN_DOT3STATSEXCESSIVECOLLISIONS_FLAG;
         }
+        if (DOT3STATSDEFERREDTRANSMISSIONS(s)) {
+            data->dot3StatsDeferredTransmissions = (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_DOT3STATSDEFERREDTRANSMISSIONS_FLAG;
+        }
     }
 
     free(eth_strings);
