@@ -11,9 +11,10 @@
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/data_access/swinst.h>
-
 #include <stdlib.h>
 #include <unistd.h>
+#include "swinst.h"
+#include "swinst_private.h"
 
 netsnmp_feature_child_of(software_installed, libnetsnmpmibs)
 
@@ -24,10 +25,6 @@ netsnmp_feature_child_of(swinst_entry_remove, netsnmp_unused)
 
 static void netsnmp_swinst_entry_free_cb(netsnmp_swinst_entry *, void *);
 
-extern void netsnmp_swinst_arch_init(void);
-extern void netsnmp_swinst_arch_shutdown(void);
-extern int netsnmp_swinst_arch_load(netsnmp_container *, u_int);
-
 void init_swinst( void )
 {
     static int initialized = 0;
