@@ -15,6 +15,8 @@
 #include "ip-forward-mib/data_access/route_ioctl.h"
 #include "ip-forward-mib/inetCidrRouteTable/inetCidrRouteTable_constants.h"
 #include "if-mib/data_access/interface_ioctl.h"
+#include "route.h"
+#include "route_private.h"
 
 static int
 _type_from_flags(unsigned int flags)
@@ -215,7 +217,6 @@ _load_ipv6(netsnmp_container* container, u_long *index )
         return -2;
     }
     
-    fgets(line,sizeof(line),in); /* skip header */
     while (fgets(line, sizeof(line), in)) {
         char            c_name[IFNAMSIZ+1];
         char            c_dest[33], c_src[33], c_next[33];
