@@ -7,8 +7,11 @@
 
 
 config_require(mibII/udpTable)
-config_arch_require(solaris2, kernel_sunos5)
-config_arch_require(linux, mibII/kernel_linux)
+
+config_arch_require(solaris2,        kernel_sunos5)
+config_arch_require(linux,     mibII/kernel_linux)
+config_arch_require(netbsd,    mibII/kernel_netbsd)
+config_arch_require(netbsdelf, mibII/kernel_netbsd)
 
 extern void     init_udp(void);
 extern Netsnmp_Node_Handler udp_handler;
