@@ -45,6 +45,7 @@ PERFORMANCE OF THIS SOFTWARE.
  * (simon@switch.ch) 1997
  */
 
+
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-features.h>
 
@@ -64,6 +65,10 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "struct.h"
 #include "util_funcs.h"
 
+#if defined(cygwin) || defined(mingw32)
+#include <winerror.h>
+#endif
+
 netsnmp_feature_child_of(get_routes, libnetsnmpmibs)
 
 #ifndef  MIN
@@ -75,8 +80,6 @@ netsnmp_feature_child_of(get_routes, libnetsnmpmibs)
 #include <netinet/mib_kern.h>
 #endif                          /* hpux */
 
-extern WriteMethod write_rte;
-
 #if !defined (WIN32) && !defined (cygwin)
 
 #ifdef USE_SYSCTL_ROUTE_DUMP
@@ -87,9 +90,6 @@ static unsigned char *all_routes = 0;
 static unsigned char *all_routes_end;
 static size_t   all_routes_size;
 
-extern const struct sockaddr *get_address(const void *, int, int);
-extern const struct in_addr *get_in_address(const void *, int, int);
-
 /*
  * var_ipRouteEntry(...
  * Arguments:
@@ -772,6 +772,8 @@ var_ipRouteEntry(struct variable * vp,
         if(entry.ipRouteInfo.re_ire_type&IRE_CACHE)
             continue;
 #endif /* HAVE_DEFINED_IRE_CACHE */
+        if(entry.ipRouteInfo.re_ire_type & IRE_BROADCAST)
+            continue;
         COPY_IPADDR(cp, (u_char *) & entry.ipRouteDest, op,
                     current + IP_ROUTEADDR_OFF);
         if (exact) {
