@@ -242,10 +242,70 @@ interface_ioctl_etherstats_get (etherStatsTable_rowreq_ctx *rowreq_ctx , int fd,
         strlcpy(s, (const char *) &eth_strings->data[i * ETH_GSTRING_LEN],
                 sizeof(s));
         
+        if (ETHERSTATSOCTETS(s)) {
+            data->etherStatsOctets += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSOCTETS_FLAG;
+        }
+        if (ETHERSTATSPKTS(s)) {
+            data->etherStatsPkts += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSPKTS_FLAG;
+        }
+        if (ETHERSTATSBROADCASTPKTS(s)) {
+            data->etherStatsBroadcastPkts += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSBROADCASTPKTS_FLAG;
+        }
+        if (ETHERSTATSMULTICASTPKTS(s)) {
+            data->etherStatsMulticastPkts = (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSMULTICASTPKTS_FLAG;
+        }
+        if (ETHERSTATSCRCALIGNERRORS(s)) {
+            data->etherStatsCRCAlignErrors += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSCRCALIGNERRORS_FLAG;
+        }
+        if (ETHERSTATSUNDERSIZEPKTS(s)) {
+            data->etherStatsUndersizePkts += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSUNDERSIZEPKTS_FLAG;
+        }
+        if (ETHERSTATSOVERSIZEPKTS(s)) {
+            data->etherStatsOversizePkts += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSOVERSIZEPKTS_FLAG;
+        }
+        if (ETHERSTATSFRAGMENTS(s)) {
+            data->etherStatsFragments += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSFRAGMENTS_FLAG;
+        }
         if (ETHERSTATSJABBERS(s)) {
-            data->etherStatsJabbers = (u_long)eth_stats->data[i];
+            data->etherStatsJabbers += (u_long)eth_stats->data[i];
             rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSJABBERS_FLAG;
         }
+        if (ETHERSTATSCOLLISIONS(s)) {
+            data->etherStatsCollisions += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSCOLLISIONS_FLAG;
+        }
+        if (ETHERSTATSPKTS64OCTETS(s)) {
+            data->etherStatsPkts64Octets += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSPKTS64OCTETS_FLAG;
+        }
+        if (ETHERSTATSPKTS65TO127OCTETS(s)) {
+            data->etherStatsPkts65to127Octets += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSPKTS65TO127OCTETS_FLAG;
+        }
+        if (ETHERSTATSPKTS128TO255OCTETS(s)) {
+            data->etherStatsPkts128to255Octets += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSPKTS128TO255OCTETS_FLAG;
+        }
+        if (ETHERSTATSPKTS256TO511OCTETS(s)) {
+            data->etherStatsPkts256to511Octets += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSPKTS256TO511OCTETS_FLAG;
+        }
+        if (ETHERSTATSPKTS512TO1023OCTETS(s)) {
+            data->etherStatsPkts512to1023Octets += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSPKTS512TO1023OCTETS_FLAG;
+        }
+        if (ETHERSTATSPKTS1024TO1518OCTETS(s)) {
+            data->etherStatsPkts1024to1518Octets += (u_long)eth_stats->data[i];
+            rowreq_ctx->column_exists_flags |= COLUMN_ETHERSTATSPKTS1024TO1518OCTETS_FLAG;
+        }
     }
     free(eth_strings);
     free(eth_stats);
