@@ -1,3 +1,9 @@
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-features.h>
 
@@ -77,13 +83,13 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
         /*
          * check tag list to see if we match 
          */
-        if (targaddrs->tagList) {
+        if (targaddrs->tagListData) {
             int matched = 0;
 
             /*
              * loop through tag list looking for requested tags 
              */
-            for (cp = targaddrs->tagList; cp && !matched;) {
+            for (cp = targaddrs->tagListData; cp && !matched;) {
                 cp = copy_nword(cp, buf, sizeof(buf));
                 for (i = 0; i < numtags && !matched; i++) {
                     if (strcmp(buf, tags[i]) == 0) {
@@ -93,8 +99,9 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
                         DEBUGMSGTL(("target_sessions", "found one: %s\n",
                                     tags[i]));
 
-                        if (targaddrs->params) {
-                            param = get_paramEntry(targaddrs->params);
+                        if (targaddrs->paramsData) {
+                            param = get_paramEntry2(targaddrs->paramsData,
+                                                    targaddrs->paramsLen);
                             if (!param
                                 || param->rowStatus != SNMP_ROW_ACTIVE) {
                                 /*
@@ -198,14 +205,15 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
                                 netsnmp_cert *cert;
                                 char         *server_id = NULL;
                                 char	      buf[33];
+                                int           len;
 
                                 DEBUGMSGTL(("target_sessions",
                                             "  looking up our id: %s\n",
-                                            targaddrs->params));
+                                            targaddrs->paramsData));
                                 cert =
                                     netsnmp_cert_find(NS_CERT_IDENTITY,
                                                       NS_CERTKEY_TARGET_PARAM,
-                                                      targaddrs->params);
+                                                      targaddrs->paramsData);
                                 netsnmp_assert(t->f_config);
                                 if (cert) {
                                     DEBUGMSGTL(("target_sessions",
@@ -214,9 +222,10 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
                                     t->f_config(t, "localCert",
                                                 cert->fingerprint);
                                 }
-                                memcpy(buf, targaddrs->nameData,
-                                       targaddrs->nameLen);
-                                buf[targaddrs->nameLen] = '\0';
+                                len = targaddrs->nameLen >= sizeof(buf) ?
+                                    sizeof(buf) - 1 : targaddrs->nameLen;
+                                memcpy(buf, targaddrs->nameData, len);
+                                buf[len] = '\0';
                                 DEBUGMSGTL(("target_sessions",
                                             "  looking up their id: %s\n",
                                             buf));
@@ -242,7 +251,7 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
                                 }
                             }
 #endif
-                            memset(&thissess, 0, sizeof(thissess));
+                            snmp_sess_init(&thissess);
                             thissess.timeout = (targaddrs->timeout) * 10000;
                             thissess.retries = targaddrs->retryCount;
                             DEBUGMSGTL(("target_sessions",
@@ -263,20 +272,23 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
                                 netsnmp_transport_free(t);
                                 continue;
                             }
-                            thissess.paramName = strdup(param->paramName);
+                            thissess.paramName =
+                                netsnmp_memdup_nt(param->paramNameData,
+                                                  param->paramNameLen, NULL);
                             thissess.version = param->mpModel;
                             if (param->mpModel == SNMP_VERSION_3) {
-                                thissess.securityName = strdup(param->secName);
-                                thissess.securityNameLen =
-                                    strlen(thissess.securityName);
+                                thissess.securityName =
+                                    netsnmp_memdup_nt(param->secNameData,
+                                                      param->secNameLen,
+                                                      &thissess.securityNameLen);
                                 thissess.securityLevel = param->secLevel;
                                 thissess.securityModel = param->secModel;
 #if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
                             } else {
                                 thissess.community =
-                                    (u_char *) strdup(param->secName);
-                                thissess.community_len =
-                                    strlen((char *) thissess.community);
+                                    netsnmp_memdup_nt(param->secNameData,
+                                                      param->secNameLen,
+                                                      &thissess.community_len);
 #endif
                             }
 
@@ -289,7 +301,9 @@ get_target_sessions(char *taglist, TargetFilterFunction * filterfunct,
                         if (targaddrs->sess) {
                             if (NULL == targaddrs->sess->paramName)
                                 targaddrs->sess->paramName =
-                                    strdup(param->paramName);
+                                    netsnmp_memdup_nt(param->paramNameData,
+                                                      param->paramNameLen,
+                                                      NULL);
 
                             targaddrs->sess->next = ret;
                             ret = targaddrs->sess;
