@@ -3,6 +3,7 @@
  */
 
 #include <net-snmp/net-snmp-config.h>
+#include <net-snmp/net-snmp-features.h>
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/library/snmp_api.h>
@@ -10,6 +11,10 @@
 
 static netsnmp_handler_registration* _myreg = NULL;
 
+netsnmp_feature_require(helper_statistics)
+
+netsnmp_feature_child_of(shutdown_snmptsmsession, netsnmp_unused)
+
 /** Initializes the snmpTsmStats module */
 void
 init_snmpTsmStats(void)
@@ -36,6 +41,7 @@ init_snmpTsmStats(void)
     }
 }
 
+#ifndef NETSNMP_FEATURE_REMOVE_SHUTDOWN_SNMPTSMSESSION
 void
 shutdown_snmpTsmSession(void)
 {
@@ -44,3 +50,4 @@ shutdown_snmpTsmSession(void)
         _myreg = NULL;
     }
 }
+#endif /* NETSNMP_FEATURE_REMOVE_SHUTDOWN_SNMPTSMSESSION */
