@@ -93,6 +93,41 @@ shutdown_pass_persist(void)
     destruct_persist_pipes();
 }
 
+
+#ifdef USING_SINGLE_COMMON_PASSPERSIST_INSTANCE
+void
+pass_persist_group(struct extensible *persistpassthrus)
+{
+   struct extensible *ptmp, *ptmp1;
+
+   /*
+    * reset groupping
+    */
+   for (ptmp = persistpassthrus; ptmp != NULL; ptmp = ptmp->next) {
+      ptmp->passpersist_inst = NULL;
+   }
+
+   /*
+    * group
+    */
+   for (ptmp = persistpassthrus; ptmp != NULL; ptmp = ptmp->next) {
+      /* skip already groupped items */
+      if (ptmp->passpersist_inst != NULL) {
+         continue;
+      }
+      for (ptmp1 = persistpassthrus; ptmp1 != NULL; ptmp1 = ptmp1->next) {
+         if (ptmp1 == ptmp) {
+            continue;
+         }
+
+         if (strcmp(ptmp->command, ptmp1->command) == 0) {
+            ptmp1->passpersist_inst = ptmp;
+         }
+      }
+   }
+}
+#endif /* USING_SINGLE_COMMON_PASSPERSIST_INSTANCE */
+
 void
 pass_persist_parse_config(const char *token, char *cptr)
 {
@@ -143,7 +178,7 @@ pass_persist_parse_config(const char *token, char *cptr)
 
     while (*ppass != NULL)
         ppass = &((*ppass)->next);
-    (*ppass) = (struct extensible *) malloc(sizeof(struct extensible));
+    *ppass = calloc(1, sizeof(**ppass));
     if (*ppass == NULL)
         return;
     (*ppass)->type = PASSTHRU_PERSIST;
@@ -155,14 +190,19 @@ pass_persist_parse_config(const char *token, char *cptr)
     /*
      * path
      */
+    free((*ppass)->command);
+    (*ppass)->command = NULL;
     cptr = skip_white(cptr);
     if (cptr == NULL) {
         config_perror("No command specified on pass_persist line");
-        (*ppass)->command[0] = 0;
+        if (asprintf(&(*ppass)->command, "%s", "") < 0) {
+        }
     } else {
         for (tcptr = cptr; *tcptr != 0 && *tcptr != '#' && *tcptr != ';';
              tcptr++);
-        sprintf((*ppass)->command, "%.*s", (int) (tcptr - cptr), cptr);
+        if (asprintf(&(*ppass)->command, "%.*s", (int)(tcptr - cptr), cptr)
+            < 0) {
+        }
     }
     strlcpy((*ppass)->name, (*ppass)->command, sizeof((*ppass)->name));
     (*ppass)->next = NULL;
@@ -195,6 +235,10 @@ pass_persist_parse_config(const char *token, char *cptr)
         ptmp->next = NULL;
         free(etmp);
     }
+
+#ifdef USING_SINGLE_COMMON_PASSPERSIST_INSTANCE
+    pass_persist_group(persistpassthrus);
+#endif /* USING_SINGLE_COMMON_PASSPERSIST_INSTANCE */
 }
 
 void
@@ -218,6 +262,28 @@ pass_persist_free_config(void)
     numpersistpassthrus = 0;
 }
 
+#ifdef USING_SINGLE_COMMON_PASSPERSIST_INSTANCE
+int get_exten_group_id(struct extensible *persistpassthru,
+                       int                current_id)
+{
+   struct extensible *ptmp;
+   int                idx;
+
+   if (persistpassthru == NULL)
+      return current_id;
+
+   for (idx = 1, ptmp = persistpassthrus;
+         ptmp != NULL; ptmp = ptmp->next, idx++) {
+      if (ptmp == persistpassthru) {
+         return idx;
+      }
+   }
+
+   /* should never really come here, but safety doesn't hurt */
+   return current_id;
+}
+#endif /* USING_SINGLE_COMMON_PASSPERSIST_INSTANCE */
+
 u_char         *
 var_extensible_pass_persist(struct variable *vp,
                             oid * name,
@@ -231,6 +297,7 @@ var_extensible_pass_persist(struct variable *vp,
     static char     buf2[SNMP_MAXBUF];
     struct extensible *persistpassthru;
     FILE           *file;
+    int             pipe_idx;
 
     /*
      * Make sure that our basic pipe structure is malloced 
@@ -252,25 +319,34 @@ var_extensible_pass_persist(struct variable *vp,
             else
                 sprint_mib_oid(buf, name, *length);
 
+            pipe_idx = i;
+#ifdef USING_SINGLE_COMMON_PASSPERSIST_INSTANCE
+            pipe_idx = 
+               get_exten_group_id(persistpassthru->passpersist_inst, i);
+
+            if (pipe_idx != i) {
+                  persistpassthru = persistpassthru->passpersist_inst;
+            }
+#endif /* USING_SINGLE_COMMON_PASSPERSIST_INSTANCE */
             /*
              * Open our pipe if necessary 
              */
-            if (!open_persist_pipe(i, persistpassthru->name)) {
+            if (!open_persist_pipe(pipe_idx, persistpassthru->name)) {
                 return (NULL);
             }
 
-            if (exact)
-                snprintf(persistpassthru->command,
-                  sizeof(persistpassthru->command), "get\n%s\n", buf);
-            else
-                snprintf(persistpassthru->command,
-                  sizeof(persistpassthru->command), "getnext\n%s\n", buf);
-            persistpassthru->command[ sizeof(persistpassthru->command)-1 ] = 0;
+            free(persistpassthru->command);
+            if (asprintf(&persistpassthru->command, "%s\n%s\n",
+                         exact ? "get" : "getnext", buf) < 0) {
+                persistpassthru->command = NULL;
+                *var_len = 0;
+                return NULL;
+            }
 
             DEBUGMSGTL(("ucd-snmp/pass_persist",
                         "persistpass-sending:\n%s",
                         persistpassthru->command));
-            if (!write_persist_pipe(i, persistpassthru->command)) {
+            if (!write_persist_pipe(pipe_idx, persistpassthru->command)) {
                 *var_len = 0;
                 /*
                  * close_persist_pipes is called in write_persist_pipe 
@@ -282,10 +358,10 @@ var_extensible_pass_persist(struct variable *vp,
              * valid call.  Exec and get output 
              */
 		
-            if ((file = persist_pipes[i].fIn)) {
+            if ((file = persist_pipes[pipe_idx].fIn)) {
                 if (fgets(buf, sizeof(buf), file) == NULL) {
                     *var_len = 0;
-                    close_persist_pipe(i);
+                    close_persist_pipe(pipe_idx);
                     return (NULL);
                 }
                 /*
@@ -315,7 +391,7 @@ var_extensible_pass_persist(struct variable *vp,
                 if (newlen == 0 || fgets(buf, sizeof(buf), file) == NULL
                     || fgets(buf2, sizeof(buf2), file) == NULL) {
                     *var_len = 0;
-                    close_persist_pipe(i);
+                    close_persist_pipe(pipe_idx);
                     return (NULL);
                 }
                 return netsnmp_internal_pass_parse(buf, buf2, var_len, vp);
@@ -341,6 +417,7 @@ setPassPersist(int action,
     struct extensible *persistpassthru;
 
     char            buf[SNMP_MAXBUF], buf2[SNMP_MAXBUF];
+    int             pipe_idx;
 
     /*
      * Make sure that our basic pipe structure is malloced 
@@ -352,6 +429,15 @@ setPassPersist(int action,
         rtest = snmp_oidtree_compare(name, name_len,
                                      persistpassthru->miboid,
                                      persistpassthru->miblen);
+        pipe_idx = i;
+#ifdef USING_SINGLE_COMMON_PASSPERSIST_INSTANCE
+        pipe_idx = 
+           get_exten_group_id(persistpassthru->passpersist_inst, i);
+
+        if (pipe_idx != i) {
+           persistpassthru = persistpassthru->passpersist_inst;
+        }
+#endif /* USING_SINGLE_COMMON_PASSPERSIST_INSTANCE */
         if (rtest <= 0) {
             if (action != ACTION)
                 return SNMP_ERR_NOERROR;
@@ -363,29 +449,29 @@ setPassPersist(int action,
                                persistpassthru->miblen);
             else
                 sprint_mib_oid(buf, name, name_len);
-            snprintf(persistpassthru->command,
-                     sizeof(persistpassthru->command), "set\n%s\n", buf);
-            persistpassthru->command[ sizeof(persistpassthru->command)-1 ] = 0;
-            netsnmp_internal_pass_set_format(buf, var_val, var_val_type, var_val_len);
-            strlcat(persistpassthru->command, buf,
-                    sizeof(persistpassthru->command));
-            persistpassthru->command[ sizeof(persistpassthru->command)-2 ] = '\n';
-            persistpassthru->command[ sizeof(persistpassthru->command)-1 ] = 0;
+            netsnmp_internal_pass_set_format(buf2, var_val, var_val_type,
+                                             var_val_len);
+            free(persistpassthru->command);
+            if (asprintf(&persistpassthru->command, "set\n%s\n%s\n", buf,
+                         buf2) < 0) {
+                persistpassthru->command = NULL;
+                return SNMP_ERR_GENERR;
+            }
 
-            if (!open_persist_pipe(i, persistpassthru->name)) {
+            if (!open_persist_pipe(pipe_idx, persistpassthru->name)) {
                 return SNMP_ERR_NOTWRITABLE;
             }
 
             DEBUGMSGTL(("ucd-snmp/pass_persist",
                         "persistpass-writing:  %s\n",
                         persistpassthru->command));
-            if (!write_persist_pipe(i, persistpassthru->command)) {
-                close_persist_pipe(i);
+            if (!write_persist_pipe(pipe_idx, persistpassthru->command)) {
+                close_persist_pipe(pipe_idx);
                 return SNMP_ERR_NOTWRITABLE;
             }
 
-            if (fgets(buf, sizeof(buf), persist_pipes[i].fIn) == NULL) {
-                close_persist_pipe(i);
+            if (fgets(buf, sizeof(buf), persist_pipes[pipe_idx].fIn) == NULL) {
+                close_persist_pipe(pipe_idx);
                 return SNMP_ERR_NOTWRITABLE;
             }
 
@@ -580,6 +666,7 @@ open_persist_pipe(int iindex, char *command)
             recurse = 0;
             return 0;
         }
+
         if (strncmp(buf, "PONG", 4)) {
             DEBUGMSGTL(("ucd-snmp/pass_persist",
                         "open_persist_pipe: Got %s instead of PONG!\n", buf));
