@@ -113,6 +113,9 @@
 #include <netdb.h>
 #endif
 #include <errno.h>
+#if HAVE_IO_H
+#include <io.h>
+#endif
 
 #if HAVE_DIRENT_H
 # include <dirent.h>
@@ -150,7 +153,6 @@ netsnmp_feature_child_of(read_config_all, libnetsnmp)
 
 netsnmp_feature_child_of(unregister_app_config_handler, read_config_all)
 netsnmp_feature_child_of(read_config_register_app_prenetsnmp_mib_handler, netsnmp_unused)
-netsnmp_feature_child_of(read_config_register_const_config_handler, netsnmp_unused)
 
 static int      config_errors;
 
@@ -314,7 +316,6 @@ register_config_handler(const char *type,
 					    help, NORMAL_CONFIG);
 }
 
-#ifndef NETSNMP_FEATURE_REMOVE_READ_CONFIG_REGISTER_CONST_CONFIG_HANDLER
 struct config_line *
 register_const_config_handler(const char *type,
                               const char *token,
@@ -326,7 +327,6 @@ register_const_config_handler(const char *type,
                                             parser, releaser,
 					    help, NORMAL_CONFIG);
 }
-#endif /* NETSNMP_FEATURE_REMOVE_READ_CONFIG_REGISTER_CONST_CONFIG_HANDLER */
 
 struct config_line *
 register_app_config_handler(const char *token,
@@ -1020,6 +1020,8 @@ read_configs_optional(const char *optional_config, int when)
                 "reading optional configuration tokens for %s\n", type));
     
     newp = strdup(optional_config);      /* strtok_r messes it up */
+    if (!newp)
+        return ret;
     cp = strtok_r(newp, ",", &st);
     while (cp) {
         struct stat     statbuf;
@@ -1538,9 +1540,31 @@ read_config_store(const char *type, const char *line)
         if (line[strlen(line)] != '\n')
             fprintf(fout, "\n");
         DEBUGMSGTL(("read_config:store", "storing: %s\n", line));
+        fflush(fout);
+#if defined(HAVE_FSYNC)
+        fsync(fileno(fout));
+#elif defined(WIN32) && !defined(cygwin)
+        {
+            int fd;
+            void *h;
+
+            fd = _fileno(fout);
+            netsnmp_assert(fd != -1);
+            h = _get_osfhandle(fd);
+            netsnmp_assert(h != INVALID_HANDLE_VALUE);
+            FlushFileBuffers(h);
+        }
+#endif
         fclose(fout);
     } else {
-        snmp_log(LOG_ERR, "read_config_store open failure on %s\n", filep);
+        if (strcmp(NETSNMP_APPLICATION_CONFIG_TYPE, type) != 0) {
+            /*
+             * Ignore this error in client utilities, they can run with random
+             * UID/GID and typically cannot write to /var. Error message just
+             * confuses people.
+             */
+            snmp_log(LOG_ERR, "read_config_store open failure on %s\n", filep);
+        }
     }
 #ifdef NETSNMP_PERSISTENT_MASK
     umask(oldmask);
@@ -2270,10 +2294,10 @@ read_config_read_memory(int type, char *readfrom,
         return readfrom;
 
     case ASN_COUNTER64:
-        if (*len < sizeof(U64))
+        if (*len < sizeof(struct counter64))
             return NULL;
-        *len = sizeof(U64);
-        read64((U64 *) dataptr, readfrom);
+        *len = sizeof(struct counter64);
+        read64((struct counter64 *) dataptr, readfrom);
         readfrom = skip_token(readfrom);
         return readfrom;
     }
