@@ -10,6 +10,11 @@
  * Copyright @ 2003 Sun Microsystems, Inc. All rights reserved.
  * Use is subject to license terms specified in the COPYING file
  * distributed with the Net-SNMP package.
+ *
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
  */
 
 #include <net-snmp/net-snmp-config.h>
@@ -59,6 +64,7 @@
 #include <net-snmp/types.h>
 #include <net-snmp/output_api.h>
 #include <net-snmp/config_api.h>
+#include <net-snmp/library/snmpv3.h>
 #include <net-snmp/library/snmp_parse_args.h>   /* for "internal" definitions */
 #include <net-snmp/utilities.h>
 
@@ -73,8 +79,6 @@
 #include <net-snmp/library/snmpv3.h>
 #include <net-snmp/library/transform_oids.h>
 
-int             random_access = 0;
-
 void
 snmp_parse_args_usage(FILE * outf)
 {
@@ -99,7 +103,7 @@ snmp_parse_args_descriptions(FILE * outf)
 #endif /* support for community based SNMP */
     fprintf(outf, "SNMP Version 3 specific\n");
     fprintf(outf,
-            "  -a PROTOCOL\t\tset authentication protocol (MD5|SHA)\n");
+            "  -a PROTOCOL\t\tset authentication protocol (MD5|SHA|SHA-224|SHA-256|SHA-384|SHA-512)\n");
     fprintf(outf,
             "  -A PASSPHRASE\t\tset authentication protocol pass phrase\n");
     fprintf(outf,
@@ -111,7 +115,11 @@ snmp_parse_args_descriptions(FILE * outf)
     fprintf(outf, "  -n CONTEXT\t\tset context name (e.g. bridge1)\n");
     fprintf(outf, "  -u USER-NAME\t\tset security name (e.g. bert)\n");
 #ifdef HAVE_AES
-    fprintf(outf, "  -x PROTOCOL\t\tset privacy protocol (DES|AES)\n");
+    fprintf(outf, "  -x PROTOCOL\t\tset privacy protocol (DES|AES"
+#ifdef NETSNMP_DRAFT_BLUMENTHAL_AES_04
+            "|AES-192|AES-256"
+#endif
+            ")\n");
 #else
     fprintf(outf, "  -x PROTOCOL\t\tset privacy protocol (DES)\n");
 #endif
@@ -124,13 +132,15 @@ snmp_parse_args_descriptions(FILE * outf)
             "  -t TIMEOUT\t\tset the request timeout (in seconds)\n");
     fprintf(outf, "Debugging\n");
     fprintf(outf, "  -d\t\t\tdump input/output packets in hexadecimal\n");
+#ifndef NETSNMP_DISABLE_DEBUGGING
     fprintf(outf,
             "  -D[TOKEN[,...]]\tturn on debugging output for the specified TOKENs\n\t\t\t   (ALL gives extremely verbose debugging output)\n");
+#endif
     fprintf(outf, "General options\n");
     fprintf(outf,
-            "  -m MIB[:...]\t\tload given list of MIBs (ALL loads everything)\n");
+            "  -m MIB[" ENV_SEPARATOR "...]\t\tload given list of MIBs (ALL loads everything)\n");
     fprintf(outf,
-            "  -M DIR[:...]\t\tlook in given list of directories for MIBs\n");
+            "  -M DIR[" ENV_SEPARATOR "...]\t\tlook in given list of directories for MIBs\n");
 #ifndef NETSNMP_DISABLE_MIB_LOADING
     fprintf(outf,
             "    (default: %s)\n", netsnmp_get_mib_directory());
@@ -184,10 +194,6 @@ handle_long_opt(const char *myoptarg)
     free(cp);
 }
 
-extern int      snmpv3_options(char *optarg, netsnmp_session * session,
-                               char **Apsz, char **Xpsz, int argc,
-                               char *const *argv);
-
 int
 netsnmp_parse_args(int argc,
                    char **argv,
@@ -195,22 +201,27 @@ netsnmp_parse_args(int argc,
                    void (*proc) (int, char *const *, int),
                    int flags)
 {
-    static char	   *sensitive[4] = { NULL, NULL, NULL, NULL };
-    int             arg, sp = 0, testcase = 0;
+    int             arg, sp = 0;
     char           *cp;
     char           *Apsz = NULL;
     char           *Xpsz = NULL;
     char           *Cpsz = NULL;
     char            Opts[BUF_SIZE];
     int             zero_sensitive = !( flags & NETSNMP_PARSE_ARGS_NOZERO );
+    char           *backup_NETSNMP_DS_LIB_OUTPUT_PRECISION = NULL;
 
     /*
      * initialize session to default values 
      */
     snmp_sess_init(session);
-    strcpy(Opts, "Y:VhHm:M:O:I:P:D:dv:r:t:c:Z:e:E:n:u:l:x:X:a:A:p:T:-:3:s:S:L:");
-    if (localOpts)
+    strcpy(Opts, "Y:VhHm:M:O:I:P:D:dv:r:t:c:Z:e:E:n:u:l:x:X:a:A:p:T:-:3:L:s:");
+    if (localOpts) {
+        if (strlen(localOpts) + strlen(Opts) >= sizeof(Opts)) {
+            snmp_log(LOG_ERR, "Too many localOpts in snmp_parse_args()\n");
+            return -1;
+        }
         strcat(Opts, localOpts);
+    }
 
     /*
      * get the options 
@@ -222,7 +233,8 @@ netsnmp_parse_args(int argc,
 
     optind = 1;
     while ((arg = getopt(argc, argv, Opts)) != EOF) {
-        DEBUGMSGTL(("snmp_parse_args", "handling (#%d): %c\n", optind, arg));
+        DEBUGMSGTL(("snmp_parse_args", "handling (#%d): %c (optarg %s) (sp %d)\n",
+                    optind, arg, optarg, sp));
         switch (arg) {
         case '-':
             if (strcasecmp(optarg, "help") == 0) {
@@ -245,7 +257,7 @@ netsnmp_parse_args(int argc,
             break;
 
         case 'H':
-            init_snmp("snmpapp");
+            init_snmp(NETSNMP_APPLICATION_CONFIG_TYPE);
             fprintf(stderr, "Configuration directives understood:\n");
             read_config_print_usage("  ");
             return (NETSNMP_PARSE_ARGS_SUCCESS_EXIT);
@@ -266,7 +278,7 @@ netsnmp_parse_args(int argc,
 #endif /* NETSNMP_DISABLE_MIB_LOADING */
 
         case 'O':
-            cp = snmp_out_toggle_options(optarg);
+            cp = snmp_out_options(optarg, argc, argv);
             if (cp != NULL) {
                 fprintf(stderr, "Unknown output option passed to -O: %c.\n", 
 			*cp);
@@ -295,8 +307,13 @@ netsnmp_parse_args(int argc,
 #endif /* NETSNMP_DISABLE_MIB_LOADING */
 
         case 'D':
+#ifdef NETSNMP_NO_DEBUGGING
+            fprintf(stderr, "Debug not configured in\n");
+            return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
+#else
             debug_register_tokens(optarg);
             snmp_set_do_debugging(1);
+#endif
             break;
 
         case 'd':
@@ -304,6 +321,10 @@ netsnmp_parse_args(int argc,
 				   NETSNMP_DS_LIB_DUMP_PACKET, 1);
             break;
 
+        case 's':
+            session->localname = strdup(optarg);
+            break;
+
         case 'v':
             session->version = -1;
 #ifndef NETSNMP_DISABLE_SNMPV1
@@ -343,6 +364,7 @@ netsnmp_parse_args(int argc,
             tmpcp = strchr(tmpopt, '=');
             if (!tmpcp) {
                 fprintf(stderr, "-T expects a NAME=VALUE pair.\n");
+                free(tmpopt);
                 return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
             }
             *tmpcp++ = '\0';
@@ -364,10 +386,8 @@ netsnmp_parse_args(int argc,
             }
 
             /* set the config */
-            strncpy(leftside, tmpopt, sizeof(leftside));
-            leftside[sizeof(leftside)-1] = '0';
-            strncpy(rightside, tmpcp, sizeof(rightside));
-            rightside[sizeof(rightside)-1] = '0';
+            strlcpy(leftside, tmpopt, sizeof(leftside));
+            strlcpy(rightside, tmpcp, sizeof(rightside));
 
             CONTAINER_INSERT(session->transport_configuration,
                              netsnmp_transport_create_config(leftside,
@@ -394,10 +414,9 @@ netsnmp_parse_args(int argc,
 
         case 'c':
 	    if (zero_sensitive) {
-		if ((sensitive[sp] = strdup(optarg)) != NULL) {
-		    Cpsz = sensitive[sp];
+                SNMP_FREE(Cpsz); /* free any previous value */
+		if ((Cpsz = strdup(optarg)) != NULL) {
 		    memset(optarg, '\0', strlen(optarg));
-		    sp++;
 		} else {
 		    fprintf(stderr, "malloc failure processing -c flag.\n");
 		    return NETSNMP_PARSE_ARGS_ERROR;
@@ -408,8 +427,8 @@ netsnmp_parse_args(int argc,
             break;
 
         case '3':
-	    /*  TODO: This needs to zero things too.  */
-            if (snmpv3_options(optarg, session, &Apsz, &Xpsz, argc, argv) < 0){
+            if (snmpv3_parse_args(optarg, session, &Apsz, &Xpsz, argc, argv,
+                                  flags) < 0){
                 return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
             }
             break;
@@ -423,202 +442,22 @@ netsnmp_parse_args(int argc,
 #define SNMPV3_CMD_OPTIONS
 #ifdef  SNMPV3_CMD_OPTIONS
         case 'Z':
-            errno = 0;
-            session->engineBoots = strtoul(optarg, &cp, 10);
-            if (errno || cp == optarg) {
-                fprintf(stderr, "Need engine boots value after -Z flag.\n");
-                return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-            }
-            if (*cp == ',') {
-                char *endptr;
-                cp++;
-                session->engineTime = strtoul(cp, &endptr, 10);
-                if (errno || cp == endptr) {
-                    fprintf(stderr, "Need engine time after \"-Z engineBoot,\".\n");
-                    return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-                }
-            }
-            /*
-             * Handle previous '-Z boot time' syntax 
-             */
-            else if (optind < argc) {
-                session->engineTime = strtoul(argv[optind], &cp, 10);
-                if (errno || cp == argv[optind]) {
-                    fprintf(stderr, "Need engine time after \"-Z engineBoot\".\n");
-                    return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-                }
-            } else {
-                fprintf(stderr, "Need engine time after \"-Z engineBoot\".\n");
-                return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-            }
-            break;
-
-        case 'e':{
-                size_t ebuf_len = 32, eout_len = 0;
-                u_char *ebuf = (u_char *)malloc(ebuf_len);
-
-                if (ebuf == NULL) {
-                    fprintf(stderr, "malloc failure processing -e flag.\n");
-                    return (NETSNMP_PARSE_ARGS_ERROR);
-                }
-                if (!snmp_hex_to_binary
-                    (&ebuf, &ebuf_len, &eout_len, 1, optarg)) {
-                    fprintf(stderr, "Bad engine ID value after -e flag.\n");
-                    free(ebuf);
-                    return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-                }
-                if ((eout_len < 5) || (eout_len > 32)) {
-                    fprintf(stderr, "Invalid engine ID value after -e flag.\n");
-                    free(ebuf);
-                    return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-                }
-                session->securityEngineID = ebuf;
-                session->securityEngineIDLen = eout_len;
-                break;
-            }
-
-        case 'E':{
-                size_t ebuf_len = 32, eout_len = 0;
-                u_char *ebuf = (u_char *)malloc(ebuf_len);
-
-                if (ebuf == NULL) {
-                    fprintf(stderr, "malloc failure processing -E flag.\n");
-                    return (NETSNMP_PARSE_ARGS_ERROR);
-                }
-                if (!snmp_hex_to_binary(&ebuf, &ebuf_len,
-					&eout_len, 1, optarg)) {
-                    fprintf(stderr, "Bad engine ID value after -E flag.\n");
-                    free(ebuf);
-                    return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-                }
-                if ((eout_len < 5) || (eout_len > 32)) {
-                    fprintf(stderr, "Invalid engine ID value after -E flag.\n");
-                    free(ebuf);
-                    return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-                }
-                session->contextEngineID = ebuf;
-                session->contextEngineIDLen = eout_len;
-                break;
-            }
-
+        case 'e':
+        case 'E':
         case 'n':
-            session->contextName = optarg;
-            session->contextNameLen = strlen(optarg);
-            break;
-
-        case 'u':
-	    if (zero_sensitive) {
-		if ((sensitive[sp] = strdup(optarg)) != NULL) {
-		    session->securityName = sensitive[sp];
-		    session->securityNameLen = strlen(sensitive[sp]);
-		    memset(optarg, '\0', strlen(optarg));
-		    sp++;
-		} else {
-		    fprintf(stderr, "malloc failure processing -u flag.\n");
-		    return NETSNMP_PARSE_ARGS_ERROR;
-		}
-	    } else {
-		session->securityName = optarg;
-		session->securityNameLen = strlen(optarg);
-	    }
-            break;
-
         case 'l':
-            if (!strcasecmp(optarg, "noAuthNoPriv") || !strcmp(optarg, "1")
-                || !strcasecmp(optarg, "noauth")
-                || !strcasecmp(optarg, "nanp")) {
-                session->securityLevel = SNMP_SEC_LEVEL_NOAUTH;
-            } else if (!strcasecmp(optarg, "authNoPriv")
-                       || !strcmp(optarg, "2")
-                       || !strcasecmp(optarg, "auth")
-                       || !strcasecmp(optarg, "anp")) {
-                session->securityLevel = SNMP_SEC_LEVEL_AUTHNOPRIV;
-            } else if (!strcasecmp(optarg, "authPriv")
-                       || !strcmp(optarg, "3")
-                       || !strcasecmp(optarg, "priv")
-                       || !strcasecmp(optarg, "ap")) {
-                session->securityLevel = SNMP_SEC_LEVEL_AUTHPRIV;
-            } else {
-                fprintf(stderr,
-                        "Invalid security level specified after -l flag: %s\n",
-                        optarg);
-                return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-            }
-
-            break;
-
+        case 'u':
+#ifdef NETSNMP_SECMOD_USM
         case 'a':
-#ifndef NETSNMP_DISABLE_MD5
-            if (!strcasecmp(optarg, "MD5")) {
-                session->securityAuthProto = usmHMACMD5AuthProtocol;
-                session->securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
-            } else
-#endif
-                if (!strcasecmp(optarg, "SHA")) {
-                session->securityAuthProto = usmHMACSHA1AuthProtocol;
-                session->securityAuthProtoLen = USM_AUTH_PROTO_SHA_LEN;
-            } else {
-                fprintf(stderr,
-                        "Invalid authentication protocol specified after -a flag: %s\n",
-                        optarg);
-                return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
-            }
-            break;
-
         case 'x':
-            testcase = 0;
-#ifndef NETSNMP_DISABLE_DES
-            if (!strcasecmp(optarg, "DES")) {
-                testcase = 1;
-                session->securityPrivProto = usmDESPrivProtocol;
-                session->securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
-            }
-#endif
-#ifdef HAVE_AES
-            if (!strcasecmp(optarg, "AES128") ||
-                !strcasecmp(optarg, "AES")) {
-                testcase = 1;
-                session->securityPrivProto = usmAESPrivProtocol;
-                session->securityPrivProtoLen = USM_PRIV_PROTO_AES_LEN;
-            }
-#endif
-            if (testcase == 0) {
-                fprintf(stderr,
-                      "Invalid privacy protocol specified after -x flag: %s\n",
-                        optarg);
+        case 'A':
+        case 'X':
+#endif             /* NETSNMP_SECMOD_USM */
+            if (snmpv3_parse_arg(arg, optarg, session, &Apsz, &Xpsz, argc,
+                                 argv, flags) < 0){
                 return (NETSNMP_PARSE_ARGS_ERROR_USAGE);
             }
             break;
-
-        case 'A':
-	    if (zero_sensitive) {
-		if ((sensitive[sp] = strdup(optarg)) != NULL) {
-		    Apsz = sensitive[sp];
-		    memset(optarg, '\0', strlen(optarg));
-		    sp++;
-		} else {
-		    fprintf(stderr, "malloc failure processing -A flag.\n");
-		    return NETSNMP_PARSE_ARGS_ERROR;
-		}
-	    } else {
-		Apsz = optarg;
-	    }
-            break;
-
-        case 'X':
-	    if (zero_sensitive) {
-		if ((sensitive[sp] = strdup(optarg)) != NULL) {
-		    Xpsz = sensitive[sp];
-		    memset(optarg, '\0', strlen(optarg));
-		    sp++;
-		} else {
-		    fprintf(stderr, "malloc failure processing -X flag.\n");
-		    return NETSNMP_PARSE_ARGS_ERROR;
-		}
-	    } else {
-		Xpsz = optarg;
-	    }
-            break;
 #endif                          /* SNMPV3_CMD_OPTIONS */
 
         case '?':
@@ -633,9 +472,26 @@ netsnmp_parse_args(int argc,
     DEBUGMSGTL(("snmp_parse_args", "finished: %d/%d\n", optind, argc));
     
     /*
-     * read in MIB database and initialize the snmp library
+     * save command line parameters which should have precedence above config file settings
+     *    (There ought to be a more scalable approach than this....)
      */
-    init_snmp("snmpapp");
+    if (netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_OUTPUT_PRECISION)) {
+        backup_NETSNMP_DS_LIB_OUTPUT_PRECISION = 
+            strdup(netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_OUTPUT_PRECISION));
+    }
+
+    /*
+     * read in MIB database and initialize the snmp library, read the config file
+     */
+    init_snmp(NETSNMP_APPLICATION_CONFIG_TYPE);
+
+    /*
+     * restore command line parameters which should have precedence above config file settings
+     */
+    if(backup_NETSNMP_DS_LIB_OUTPUT_PRECISION) {
+        netsnmp_ds_set_string(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_OUTPUT_PRECISION, backup_NETSNMP_DS_LIB_OUTPUT_PRECISION);
+        free(backup_NETSNMP_DS_LIB_OUTPUT_PRECISION);
+    }
 
     /*
      * session default version 
@@ -680,6 +536,9 @@ netsnmp_parse_args(int argc,
         }
     }
 
+#ifdef NETSNMP_SECMOD_USM
+    /* XXX: this should ideally be moved to snmpusm.c somehow */
+
     /*
      * make master key from pass phrases 
      */
@@ -695,20 +554,10 @@ netsnmp_parse_args(int argc,
                 snmp_duplicate_objid(def, session->securityAuthProtoLen);
         }
         if (session->securityAuthProto == NULL) {
-#ifndef NETSNMP_DISABLE_MD5
-            /*
-             * assume MD5
-             */
             session->securityAuthProto =
-                snmp_duplicate_objid(usmHMACMD5AuthProtocol,
-                                     USM_AUTH_PROTO_MD5_LEN);
-            session->securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
-#else
-            session->securityAuthProto =
-                snmp_duplicate_objid(usmHMACSHA1AuthProtocol,
-                                     USM_AUTH_PROTO_SHA_LEN);
-            session->securityAuthProtoLen = USM_AUTH_PROTO_SHA_LEN;
-#endif
+                snmp_duplicate_objid(SNMP_DEFAULT_AUTH_PROTO,
+                                     SNMP_DEFAULT_AUTH_PROTOLEN);
+            session->securityAuthProtoLen = SNMP_DEFAULT_AUTH_PROTOLEN;
         }
         if (generate_Ku(session->securityAuthProto,
                         session->securityAuthProtoLen,
@@ -720,6 +569,7 @@ netsnmp_parse_args(int argc,
                     "Error generating a key (Ku) from the supplied authentication pass phrase. \n");
             return (NETSNMP_PARSE_ARGS_ERROR);
         }
+        free(Apsz);
     }
     if (Xpsz) {
         session->securityPrivKeyLen = USM_PRIV_KU_LEN;
@@ -733,21 +583,10 @@ netsnmp_parse_args(int argc,
                 snmp_duplicate_objid(def, session->securityPrivProtoLen);
         }
         if (session->securityPrivProto == NULL) {
-            /*
-             * assume DES 
-             */
-#ifndef NETSNMP_DISABLE_DES
             session->securityPrivProto =
-                snmp_duplicate_objid(usmDESPrivProtocol,
-                                     USM_PRIV_PROTO_DES_LEN);
-            session->securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
-#else
-            session->securityPrivProto =
-                snmp_duplicate_objid(usmAESPrivProtocol,
-                                     USM_PRIV_PROTO_AES_LEN);
-            session->securityPrivProtoLen = USM_PRIV_PROTO_AES_LEN;
-#endif
-
+                snmp_duplicate_objid(SNMP_DEFAULT_PRIV_PROTO,
+                                     SNMP_DEFAULT_PRIV_PROTOLEN);
+            session->securityPrivProtoLen = SNMP_DEFAULT_PRIV_PROTOLEN;
         }
         if (generate_Ku(session->securityAuthProto,
                         session->securityAuthProtoLen,
@@ -759,7 +598,10 @@ netsnmp_parse_args(int argc,
                     "Error generating a key (Ku) from the supplied privacy pass phrase. \n");
             return (NETSNMP_PARSE_ARGS_ERROR);
         }
+        free(Xpsz);
     }
+#endif /* NETSNMP_SECMOD_USM */
+
     /*
      * get the hostname 
      */
