@@ -25,6 +25,7 @@ netsnmp_feature_require(sockaddr_size)
 
 #include <net-snmp/library/snmpDTLSUDPDomain.h>
 #include <net-snmp/library/snmpUDPIPv6Domain.h>
+#include <net-snmp/library/snmp_assert.h>
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -114,14 +115,16 @@ typedef struct bio_cache_s {
 
 /** bio_cache flags */
 #define NETSNMP_BIO_HAVE_COOKIE        0x0001 /* verified cookie */
-#define NETSNMP_BIO_CONNECTED          0x0002 /* recieved decoded data */
+#define NETSNMP_BIO_CONNECTED          0x0002 /* received decoded data */
 #define NETSNMP_BIO_DISCONNECTED       0x0004 /* peer shutdown */
 
 static bio_cache *biocache = NULL;
 
 static int openssl_addr_index = 0;
 
-static int netsnmp_dtls_verify_cookie(SSL *ssl, unsigned char *cookie,
+static int netsnmp_dtls_verify_cookie(SSL *ssl,
+                                      SECOND_APPVERIFY_COOKIE_CB_ARG_QUALIFIER
+                                      unsigned char *cookie,
                                       unsigned int cookie_len);
 static int netsnmp_dtls_gen_cookie(SSL *ssl, unsigned char *cookie,
                                    unsigned int *cookie_len);
@@ -131,10 +134,11 @@ static int netsnmp_dtls_gen_cookie(SSL *ssl, unsigned char *cookie,
 /* XXX: handle state issues for new connections to reduce DOS issues */
 /*      (TLS should do this, but openssl can't do more than one ctx per sock */
 /* XXX: put a timer on the cache for expirary purposes */
-static bio_cache *find_bio_cache(netsnmp_sockaddr_storage *from_addr) {
+static bio_cache *find_bio_cache(const netsnmp_sockaddr_storage *from_addr)
+{
     bio_cache *cachep = NULL;
     
-    for(cachep = biocache; cachep; cachep = cachep->next) {
+    for (cachep = biocache; cachep; cachep = cachep->next) {
 
         if (cachep->sas.sa.sa_family != from_addr->sa.sa_family)
             continue;
@@ -162,10 +166,12 @@ static bio_cache *find_bio_cache(netsnmp_sockaddr_storage *from_addr) {
 
 /* removes a single cache entry and returns SUCCESS on finding and
    removing it. */
-static int remove_bio_cache(bio_cache *thiscache) {
+static int remove_bio_cache(bio_cache *thiscache)
+{
     bio_cache *cachep = NULL, *prevcache = NULL;
+
     cachep = biocache;
-    while(cachep) {
+    while (cachep) {
         if (cachep == thiscache) {
 
             /* remove it from the list */
@@ -185,18 +191,20 @@ static int remove_bio_cache(bio_cache *thiscache) {
 }
 
 /* frees the contents of a bio_cache */
-static void free_bio_cache(bio_cache *cachep) {
+static void free_bio_cache(bio_cache *cachep)
+{
 /* These are freed by the SSL_free() call */
 /*
         BIO_free(cachep->read_bio);
         BIO_free(cachep->write_bio);
 */
     DEBUGMSGTL(("9:dtlsudp:bio_cache", "releasing %p\n", cachep));
-        SNMP_FREE(cachep->write_cache);
-        netsnmp_tlsbase_free_tlsdata(cachep->tlsdata);
+    SNMP_FREE(cachep->write_cache);
+    netsnmp_tlsbase_free_tlsdata(cachep->tlsdata);
 }
 
-static void remove_and_free_bio_cache(bio_cache *cachep) {
+static void remove_and_free_bio_cache(bio_cache *cachep)
+{
     /** no debug, remove_bio_cache does it */
     remove_bio_cache(cachep);
     free_bio_cache(cachep);
@@ -208,8 +216,9 @@ static void remove_and_free_bio_cache(bio_cache *cachep) {
 
 static bio_cache *
 start_new_cached_connection(netsnmp_transport *t,
-                            netsnmp_sockaddr_storage *remote_addr,
-                            int we_are_client) {
+                            const netsnmp_sockaddr_storage *remote_addr,
+                            int we_are_client)
+{
     bio_cache *cachep = NULL;
     _netsnmpTLSBaseData *tlsdata;
 
@@ -315,12 +324,12 @@ start_new_cached_connection(netsnmp_transport *t,
         DEBUGMSGTL(("dtlsudp",
                     "starting a new connection as a client to sock: %d\n",
                     t->sock));
-        tlsdata->ssl = SSL_new(sslctx_client_setup(DTLSv1_method(), tlsdata));
+        tlsdata->ssl = SSL_new(sslctx_client_setup(DTLS_method(), tlsdata));
 
         /* XXX: session setting 735 */
     } else {
         /* we're the server */
-        SSL_CTX *ctx = sslctx_server_setup(DTLSv1_method());
+        SSL_CTX *ctx = sslctx_server_setup(DTLS_method());
         if (!ctx) {
             BIO_free(cachep->read_bio);
             BIO_free(cachep->write_bio);
@@ -369,7 +378,7 @@ start_new_cached_connection(netsnmp_transport *t,
            failures and an unexpected presented certificate identity.
     */
     /* Implementation notes:
-       + Because we're working asyncronously the real "end" point of
+       + Because we're working asynchronously the real "end" point of
          opening a connection doesn't occur here as certificate
          verification and other things needs to happen first in the
          verify callback, etc.  See the netsnmp_dtlsudp_recv()
@@ -407,9 +416,11 @@ start_new_cached_connection(netsnmp_transport *t,
 
 static bio_cache *
 find_or_create_bio_cache(netsnmp_transport *t,
-                         netsnmp_sockaddr_storage *from_addr,
-                         int we_are_client) {
+                         const netsnmp_sockaddr_storage *from_addr,
+                         int we_are_client)
+{
     bio_cache *cachep = find_bio_cache(from_addr);
+
     if (NULL == cachep) {
         /* none found; need to start a new context */
         cachep = start_new_cached_connection(t, from_addr, we_are_client);
@@ -422,38 +433,47 @@ find_or_create_bio_cache(netsnmp_transport *t,
     return cachep;
 }
 
-static netsnmp_indexed_addr_pair *
-_extract_addr_pair(netsnmp_transport *t, void *opaque, int olen)
+static const netsnmp_indexed_addr_pair *
+_extract_addr_pair(netsnmp_transport *t, const void *opaque, int olen)
 {
-    netsnmp_indexed_addr_pair *addr_pair = NULL;
+    if (opaque) {
+        switch (olen) {
+        case sizeof(netsnmp_tmStateReference): {
+            const netsnmp_tmStateReference *tmStateRef = opaque;
 
-    if (opaque && olen == sizeof(netsnmp_tmStateReference)) {
-        netsnmp_tmStateReference *tmStateRef =
-            (netsnmp_tmStateReference *) opaque;
-
-        if (tmStateRef->have_addresses)
-            addr_pair = &(tmStateRef->addresses);
+            if (tmStateRef->have_addresses)
+                return &tmStateRef->addresses;
+            break;
+        }
+        default:
+            netsnmp_assert(0);
+        }
     }
-    if ((NULL == addr_pair) && (NULL != t)) {
-        if (t->data != NULL &&
-            t->data_length == sizeof(netsnmp_indexed_addr_pair))
-            addr_pair = (netsnmp_indexed_addr_pair *) (t->data);
-        else if (t->data != NULL &&
-                 t->data_length == sizeof(_netsnmpTLSBaseData)) {
-            _netsnmpTLSBaseData *tlsdata = (_netsnmpTLSBaseData *) t->data;
-            addr_pair = (netsnmp_indexed_addr_pair *) (tlsdata->addr);
+    if (t && t->data) {
+        switch (t->data_length) {
+        case sizeof(netsnmp_indexed_addr_pair):
+            return t->data;
+        case sizeof(_netsnmpTLSBaseData): {
+            _netsnmpTLSBaseData *tlsdata = t->data;
+
+            return tlsdata->addr;
+        }
+        default:
+            netsnmp_assert(0);
         }
     }
 
-    return addr_pair;
+    return NULL;
 }
 
-static struct sockaddr *
-_find_remote_sockaddr(netsnmp_transport *t, void *opaque, int olen, int *socklen)
+static const struct sockaddr *
+_find_remote_sockaddr(netsnmp_transport *t, const void *opaque, int olen,
+                      int *socklen)
 {
-    netsnmp_indexed_addr_pair *addr_pair = _extract_addr_pair(t, opaque, olen);
-    struct sockaddr *sa = NULL;
+    const netsnmp_indexed_addr_pair *addr_pair;
+    const struct sockaddr *sa = NULL;
 
+    addr_pair = _extract_addr_pair(t, opaque, olen);
     if (NULL == addr_pair)
         return NULL;
 
@@ -468,9 +488,10 @@ _find_remote_sockaddr(netsnmp_transport *t, void *opaque, int olen, int *socklen
  * queued packets out the UDP port
  */
 static int
-_netsnmp_send_queued_dtls_pkts(netsnmp_transport *t, bio_cache *cachep) {
+_netsnmp_send_queued_dtls_pkts(netsnmp_transport *t, bio_cache *cachep)
+{
     int outsize, rc2;
-    u_char outbuf[65535];
+    void *outbuf;
     
     DEBUGTRACETOK("9:dtlsudp");
 
@@ -478,24 +499,31 @@ _netsnmp_send_queued_dtls_pkts(netsnmp_transport *t, bio_cache *cachep) {
        buffer (ie, the packet to go out) and send it out
        the udp port manually */
 
-    outsize = BIO_read(cachep->write_bio, outbuf, sizeof(outbuf));
-    if (outsize > 0) {
+    outsize = BIO_ctrl_pending(cachep->write_bio);
+    outbuf = malloc(outsize);
+    if (outsize > 0 && outbuf) {
+        int socksize;
+        void *sa;
+
         DEBUGMSGTL(("dtlsudp", "have %d bytes to send\n", outsize));
 
-        /* should always be true. */
-        int socksize;
-        struct sockaddr *sa;
-        sa = _find_remote_sockaddr(t, NULL, 0, &socksize);
+        outsize = BIO_read(cachep->write_bio, outbuf, outsize);
+        sa = NETSNMP_REMOVE_CONST(struct sockaddr *,
+                                  _find_remote_sockaddr(t, NULL, 0, &socksize));
         if (NULL == sa)
             sa = &cachep->sas.sa;
         socksize = netsnmp_sockaddr_size(sa);
-        rc2 = t->base_transport->f_send(t, outbuf, outsize, (void**)&sa,
-                                        &socksize);
+        rc2 = t->base_transport->f_send(t, outbuf, outsize, &sa, &socksize);
         if (rc2 == -1) {
             snmp_log(LOG_ERR, "failed to send a DTLS specific packet\n");
         }
-    } else
+    } else if (outsize == 0) {
         DEBUGMSGTL(("9:dtlsudp", "have 0 bytes to send\n"));
+    } else {
+        DEBUGMSGTL(("9:dtlsudp", "buffer allocation failed\n"));
+    }
+
+    free(outbuf);
 
     return outsize;
 }
@@ -508,7 +536,8 @@ _netsnmp_send_queued_dtls_pkts(netsnmp_transport *t, bio_cache *cachep) {
 /* returns SNMPERR_SUCCESS if we succeeded in getting the data out */
 /* returns SNMPERR_GENERR if we still need more time */
 static int
-_netsnmp_bio_try_and_write_buffered(netsnmp_transport *t, bio_cache *cachep) {
+_netsnmp_bio_try_and_write_buffered(netsnmp_transport *t, bio_cache *cachep)
+{
     int rc;
     _netsnmpTLSBaseData *tlsdata;
     
@@ -544,10 +573,8 @@ _netsnmp_bio_try_and_write_buffered(netsnmp_transport *t, bio_cache *cachep) {
 
         /* If want_read/write but failed to actually send anything
            then we need to wait for the other side, so quit */
-        if ((errnum == SSL_ERROR_WANT_READ ||
-             errnum == SSL_ERROR_WANT_WRITE) &&
-            bytesout <= 0) {
-            /* we've failed; must need to wait longer */
+        if (bytesout <= 0) {
+            /* sending failed; must wait longer */
             return SNMPERR_GENERR;
         }
 
@@ -571,7 +598,8 @@ _netsnmp_bio_try_and_write_buffered(netsnmp_transport *t, bio_cache *cachep) {
 }
 
 static int
-_netsnmp_add_buffered_data(bio_cache *cachep, char *buf, size_t size) {
+_netsnmp_add_buffered_data(bio_cache *cachep, const char *buf, size_t size)
+{
     if (cachep->write_cache && cachep->write_cache_len > 0) {
         size_t newsize = cachep->write_cache_len + size;
 
@@ -605,7 +633,6 @@ netsnmp_dtlsudp_recv(netsnmp_transport *t, void *buf, int size,
 {
     int             rc = -1;
     netsnmp_indexed_addr_pair *addr_pair = NULL;
-    struct sockaddr *from;
     netsnmp_tmStateReference *tmStateRef = NULL;
     _netsnmpTLSBaseData *tlsdata;
     bio_cache *cachep;
@@ -632,14 +659,17 @@ netsnmp_dtlsudp_recv(netsnmp_transport *t, void *buf, int size,
 
     addr_pair = &tmStateRef->addresses;
     tmStateRef->have_addresses = 1;
-    from = (struct sockaddr *) &(addr_pair->remote_addr);
 
     while (rc < 0) {
-        char *opaque = NULL;
+        void *opaque = NULL;
         int olen;
-        rc = t->base_transport->f_recv(t, buf, size, (void**)&opaque, &olen);
-        if (rc > 0)
-            memcpy(from, opaque, olen);
+        rc = t->base_transport->f_recv(t, buf, size, &opaque, &olen);
+        if (rc > 0) {
+            if (olen > sizeof(*addr_pair))
+                snmp_log(LOG_ERR, "%s: from address length %d > %d\n",
+                         __func__, olen, (int)sizeof(*addr_pair));
+            memcpy(addr_pair, opaque, SNMP_MIN(sizeof(*addr_pair), olen));
+        }
         SNMP_FREE(opaque);
         if (rc < 0 && errno != EINTR) {
             break;
@@ -776,9 +806,7 @@ netsnmp_dtlsudp_recv(netsnmp_transport *t, void *buf, int size,
         /* If want_read/write but failed to actually send
            anything then we need to wait for the other side,
            so quit */
-        if ((errnum == SSL_ERROR_WANT_READ ||
-             errnum == SSL_ERROR_WANT_WRITE) &&
-            bytesout <= 0)
+        if (bytesout <= 0)
             break;
 
         /* retry reading */
@@ -1007,7 +1035,7 @@ netsnmp_dtlsudp_recv(netsnmp_transport *t, void *buf, int size,
           IN   tmStateReference    -- transport info
            )
     */
-    /* Implementation notes: those pamateres are all passed outward
+    /* Implementation notes: those parameters are all passed outward
        using the functions arguments and the return code below (the length) */
 
     return rc;
@@ -1016,17 +1044,17 @@ netsnmp_dtlsudp_recv(netsnmp_transport *t, void *buf, int size,
 
 
 static int
-netsnmp_dtlsudp_send(netsnmp_transport *t, void *buf, int size,
-		 void **opaque, int *olength)
+netsnmp_dtlsudp_send(netsnmp_transport *t, const void *buf, int size,
+                     void **opaque, int *olength)
 {
     int rc = -1;
-    netsnmp_indexed_addr_pair *addr_pair = NULL;
+    const netsnmp_indexed_addr_pair *addr_pair = NULL;
     bio_cache *cachep = NULL;
-    netsnmp_tmStateReference *tmStateRef = NULL;
-    u_char outbuf[65535];
+    const netsnmp_tmStateReference *tmStateRef = NULL;
+    void *outbuf;
     _netsnmpTLSBaseData *tlsdata = NULL;
     int socksize;
-    struct sockaddr *sa;
+    void *sa;
     
     DEBUGTRACETOK("9:dtlsudp");
     DEBUGMSGTL(("dtlsudp", "sending %d bytes\n", size));
@@ -1069,7 +1097,7 @@ netsnmp_dtlsudp_send(netsnmp_transport *t, void *buf, int size,
     */
     if (opaque != NULL && *opaque != NULL &&
         olength != NULL && *olength == sizeof(netsnmp_tmStateReference))
-        tmStateRef = (netsnmp_tmStateReference *) *opaque;
+        tmStateRef = *opaque;
 
 
     /* RFC5953: section 5.2, step 3:
@@ -1125,7 +1153,7 @@ netsnmp_dtlsudp_send(netsnmp_transport *t, void *buf, int size,
 
     tlsdata = cachep->tlsdata;
     if (NULL == tlsdata || NULL == tlsdata->ssl) {
-        /** xxx mem lean? free created bio cache? */
+        /** xxx mem leak? free created bio cache? */
         snmp_increment_statistic(STAT_TLSTM_SNMPTLSTMSESSIONNOSESSIONS);
         snmp_log(LOG_ERR, "bad tls data or ssl ptr in netsnmp_dtlsudp_send\n");
         return -1;
@@ -1141,6 +1169,14 @@ netsnmp_dtlsudp_send(netsnmp_transport *t, void *buf, int size,
         if (SNMPERR_GENERR == _netsnmp_bio_try_and_write_buffered(t, cachep)) {
             /* we still have data that can't get out in the buffer */
 
+            DEBUGIF ("9:dtlsudp") {
+                char *str = t->base_transport->f_fmtaddr(t, addr_pair,
+                                            sizeof(netsnmp_indexed_addr_pair));
+                DEBUGMSGTL(("9:dtlsudp", "cached %d bytes for %s on fd %d\n",
+                            size, str, t->sock));
+                free(str);
+            }
+
             /* add the new data to the end of the existing cache */
             if (_netsnmp_add_buffered_data(cachep, buf, size) !=
                 SNMPERR_SUCCESS) {
@@ -1151,7 +1187,7 @@ netsnmp_dtlsudp_send(netsnmp_transport *t, void *buf, int size,
     }
 
     DEBUGIF ("9:dtlsudp") {
-        char *str = t->base_transport->f_fmtaddr(t, (void *) addr_pair,
+        char *str = t->base_transport->f_fmtaddr(t, addr_pair,
                                         sizeof(netsnmp_indexed_addr_pair));
         DEBUGMSGTL(("9:dtlsudp", "send %d bytes from %p to %s on fd %d\n",
                     size, buf, str, t->sock));
@@ -1185,14 +1221,15 @@ netsnmp_dtlsudp_send(netsnmp_transport *t, void *buf, int size,
         /* If want_read/write but failed to actually send
            anything then we need to wait for the other side,
            so quit */
-        if ((errnum == SSL_ERROR_WANT_READ ||
-             errnum == SSL_ERROR_WANT_WRITE) &&
-            bytesout <= 0) {
+        if (bytesout <= 0) {
             /* We need more data written to or read from the socket
                but we're failing to do so and need to wait till the
                socket is ready again; unfortunately this means we need
                to buffer the SNMP data temporarily in the mean time */
 
+            DEBUGMSGTL(("9:dtlsudp", "cached %d bytes for fd %d\n", size,
+                        t->sock));
+
             /* remember the packet */
             if (_netsnmp_add_buffered_data(cachep, buf, size) !=
                 SNMPERR_SUCCESS) {
@@ -1201,7 +1238,7 @@ netsnmp_dtlsudp_send(netsnmp_transport *t, void *buf, int size,
                 return -1;
             }
 
-            /* exit out of the loop until we get caled again from
+            /* exit out of the loop until we get called again from
                socket data */ 
             break;
         }
@@ -1214,14 +1251,19 @@ netsnmp_dtlsudp_send(netsnmp_transport *t, void *buf, int size,
 
     /* for memory bios, we now read from openssl's write buffer (ie,
        the packet to go out) and send it out the udp port manually */
-    rc = BIO_read(cachep->write_bio, outbuf, sizeof(outbuf));
+    rc = BIO_ctrl_pending(cachep->write_bio);
     if (rc <= 0) {
         /* in theory an ok thing */
         return 0;
     }
+    outbuf = malloc(rc);
+    if (!outbuf)
+        return -1;
+    rc = BIO_read(cachep->write_bio, outbuf, rc);
     socksize = netsnmp_sockaddr_size(&cachep->sas.sa);
     sa = &cachep->sas.sa;
-    rc = t->base_transport->f_send(t, outbuf, rc, (void**)&sa, &socksize);
+    rc = t->base_transport->f_send(t, outbuf, rc, &sa, &socksize);
+    free(outbuf);
 
     return rc;
 }
@@ -1253,7 +1295,7 @@ netsnmp_dtlsudp_close(netsnmp_transport *t)
        + Our session id is stored as the t->data pointer
     */
     if (NULL != t->data && t->data_length == sizeof(_netsnmpTLSBaseData)) {
-        tlsbase = (_netsnmpTLSBaseData *) t->data;
+        tlsbase = t->data;
 
         if (tlsbase->addr)
             cachep = find_bio_cache(&tlsbase->addr->remote_addr);
@@ -1266,7 +1308,7 @@ netsnmp_dtlsudp_close(netsnmp_transport *t)
     if (NULL == cachep)
         return netsnmp_socketbase_close(t);
 
-    /* if we have any remaining packtes to send, try to send them */
+    /* if we have any remaining packets to send, try to send them */
     if (cachep->write_cache_len > 0) {
         int i = 0;
         char buf[8192];
@@ -1284,7 +1326,7 @@ netsnmp_dtlsudp_close(netsnmp_transport *t)
          * if negotiations have completed and we've received data, try and
          * send any queued packets.
          */
-        if (cachep->flags & NETSNMP_BIO_CONNECTED) {
+        if (1) {
             /* make configurable:
                - do this at all?
                - retries
@@ -1304,10 +1346,12 @@ netsnmp_dtlsudp_close(netsnmp_transport *t)
                 tv.tv_usec = 50000;
                 rc = select(t->sock+1, &readfs, NULL, NULL, &tv);
                 if (rc > 0) {
-                    /* junk recv for catching negotations still in play */
+                    /* junk recv for catching negotiations still in play */
                     opaque_len = 0;
-                    netsnmp_dtlsudp_recv(t, buf, sizeof(buf),
-                                         &opaque, &opaque_len);
+                    rc = netsnmp_dtlsudp_recv(t, buf, sizeof(buf),
+                                              &opaque, &opaque_len);
+                    DEBUGMSGTL(("dtlsudp:close",
+                                "netsnmp_dtlsudp_recv() returned %d\n", rc));
                     SNMP_FREE(opaque);
                 }
             } /* for loop */
@@ -1343,17 +1387,52 @@ netsnmp_dtlsudp_close(netsnmp_transport *t)
     return netsnmp_socketbase_close(t);
 }
 
-char *
-netsnmp_dtlsudp_fmtaddr(netsnmp_transport *t, void *data, int len)
+static char *
+netsnmp_dtlsudp_fmtaddr(netsnmp_transport *t, const void *data, int len,
+                        const char *pfx,
+                        char *(*fmt_base_addr)(const char *pfx,
+                                               netsnmp_transport *t,
+                                               const void *data, int len))
 {
-    int              sa_len;
-    struct sockaddr *sa = _find_remote_sockaddr(t, data, len, &sa_len);
-    if (sa) {
-        data = sa;
-        len = sa_len;
+    if (t && !data) {
+        data = t->data;
+        len = t->data_length;
     }
 
-    return netsnmp_ipv4_fmtaddr("DTLSUDP", t, data, len);
+    switch (data ? len : 0) {
+    case sizeof(netsnmp_indexed_addr_pair):
+        return netsnmp_ipv4_fmtaddr(pfx, t, data, len);
+    case sizeof(netsnmp_tmStateReference): {
+        const netsnmp_tmStateReference *r = data;
+        const netsnmp_indexed_addr_pair *p = &r->addresses;
+
+        return fmt_base_addr("DTLSUDP", t, p, sizeof(*p));
+    }
+    case sizeof(_netsnmpTLSBaseData): {
+        const _netsnmpTLSBaseData *b = data;
+        char *buf;
+
+        if (asprintf(&buf, "DTLSUDP: %s", b->addr_string) < 0)
+            buf = NULL;
+        return buf;
+    }
+    case 0:
+        return strdup("DTLSUDP: unknown");
+    default: {
+        char *buf;
+
+        if (asprintf(&buf, "DTLSUDP: len %d", len) < 0)
+            buf = NULL;
+        return buf;
+    }
+    }
+}
+
+static char *
+netsnmp_dtlsudp4_fmtaddr(netsnmp_transport *t, const void *data, int len)
+{
+    return netsnmp_dtlsudp_fmtaddr(t, data, len, "DTLSUDP",
+                                   netsnmp_ipv4_fmtaddr);
 }
 
 /*
@@ -1406,7 +1485,8 @@ _transport_common(netsnmp_transport *t, int local)
     t->f_config        = netsnmp_tlsbase_config;
     t->f_setup_session = netsnmp_tlsbase_session_init;
     t->f_accept        = NULL;
-    t->f_fmtaddr       = netsnmp_dtlsudp_fmtaddr;
+    t->f_fmtaddr       = netsnmp_dtlsudp4_fmtaddr;
+    t->f_get_taddr     = netsnmp_ipv4_get_taddr;
 
     t->flags = NETSNMP_TRANSPORT_FLAG_TUNNELED;
 
@@ -1414,7 +1494,7 @@ _transport_common(netsnmp_transport *t, int local)
 }
 
 netsnmp_transport *
-netsnmp_dtlsudp_transport(struct sockaddr_in *addr, int local)
+netsnmp_dtlsudp_transport(const struct sockaddr_in *addr, int local)
 {
     netsnmp_transport *t = NULL;
 
@@ -1428,7 +1508,7 @@ netsnmp_dtlsudp_transport(struct sockaddr_in *addr, int local)
 
     if (!local) {
         /* dtls needs to bind the socket for SSL_write to work */
-        if (connect(t->sock, (struct sockaddr *) addr, sizeof(*addr)) == -1)
+	if (connect(t->sock, (const struct sockaddr *)addr, sizeof(*addr)) < 0)
             snmp_log(LOG_ERR, "dtls: failed to connect\n");
     }
 
@@ -1438,17 +1518,11 @@ netsnmp_dtlsudp_transport(struct sockaddr_in *addr, int local)
 
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
 
-char *
-netsnmp_dtlsudp6_fmtaddr(netsnmp_transport *t, void *data, int len)
+static char *
+netsnmp_dtlsudp6_fmtaddr(netsnmp_transport *t, const void *data, int len)
 {
-    int              sa_len;
-    struct sockaddr *sa = _find_remote_sockaddr(t, data, len, &sa_len);
-    if (sa) {
-        data = sa;
-        len = sa_len;
-    }
-
-    return netsnmp_ipv6_fmtaddr("DTLSUDP6", t, data, len);
+    return netsnmp_dtlsudp_fmtaddr(t, data, len, "DTLSUDP6",
+                                   netsnmp_ipv6_fmtaddr);
 }
 
 /*
@@ -1458,7 +1532,7 @@ netsnmp_dtlsudp6_fmtaddr(netsnmp_transport *t, void *data, int len)
  */
 
 netsnmp_transport *
-netsnmp_dtlsudp6_transport(struct sockaddr_in6 *addr, int local)
+netsnmp_dtlsudp6_transport(const struct sockaddr_in6 *addr, int local)
 {
     netsnmp_transport *t = NULL;
 
@@ -1472,7 +1546,7 @@ netsnmp_dtlsudp6_transport(struct sockaddr_in6 *addr, int local)
 
     if (!local) {
         /* dtls needs to bind the socket for SSL_write to work */
-        if (connect(t->sock, (struct sockaddr *) addr, sizeof(*addr)) == -1)
+        if (connect(t->sock, (const struct sockaddr *)addr, sizeof(*addr)) < 0)
             snmp_log(LOG_ERR, "dtls: failed to connect\n");
     }
 
@@ -1480,6 +1554,7 @@ netsnmp_dtlsudp6_transport(struct sockaddr_in6 *addr, int local)
     /* XXX: and buf size */        
 
     t->f_fmtaddr       = netsnmp_dtlsudp6_fmtaddr;
+    t->f_get_taddr     = netsnmp_ipv6_get_taddr;
 
     return t;
 }
@@ -1510,7 +1585,7 @@ netsnmp_dtlsudp_create_tstring(const char *str, int isserver,
 
     /* see if we can extract the remote hostname */
     if (!isserver && t && t->data && str) {
-        tlsdata = (_netsnmpTLSBaseData *) t->data;
+        tlsdata = t->data;
         /* search for a : */
         if (NULL != (cp = strrchr(str, ':'))) {
             sprintf(buf, "%.*s", (int) SNMP_MIN(cp - str, sizeof(buf) - 1),
@@ -1526,34 +1601,25 @@ netsnmp_dtlsudp_create_tstring(const char *str, int isserver,
 
 
 netsnmp_transport *
-netsnmp_dtlsudp_create_ostring(const u_char * o, size_t o_len, int local)
+netsnmp_dtlsudp_create_ostring(const void *o, size_t o_len, int local)
 {
-    struct sockaddr_in addr;
+    struct sockaddr_in sin;
+    struct sockaddr_in6 sin6;
 
-    if (o_len == 6) {
-        unsigned short porttmp = (o[4] << 8) + o[5];
-        addr.sin_family = AF_INET;
-        memcpy((u_char *) & (addr.sin_addr.s_addr), o, 4);
-        addr.sin_port = htons(porttmp);
-        return netsnmp_dtlsudp_transport(&addr, local);
-    }
+    if (netsnmp_ipv4_ostring_to_sockaddr(&sin, o, o_len))
+        return netsnmp_dtlsudp_transport(&sin, local);
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
-    else if (o_len == 18) {
-        struct sockaddr_in6 addr6;
-        unsigned short porttmp = (o[16] << 8) + o[17];
-        addr6.sin6_family = AF_INET6;
-        memcpy((u_char *) & (addr6.sin6_addr.s6_addr), o, 4);
-        addr6.sin6_port = htons(porttmp);
-        return netsnmp_dtlsudp6_transport(&addr6, local);
-    }
+    else if (netsnmp_ipv6_ostring_to_sockaddr(&sin6, o, o_len))
+        return netsnmp_dtlsudp6_transport(&sin6, local);
 #endif
-    return NULL;
+    else
+        return NULL;
 }
 
 void
 netsnmp_dtlsudp_ctor(void)
 {
-    char indexname[] = "_netsnmp_addr_info";
+    static const char indexname[] = "_netsnmp_addr_info";
     static const char *prefixes[] = { "dtlsudp", "dtls"
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
                                       , "dtlsudp6", "dtls6"
@@ -1561,7 +1627,7 @@ netsnmp_dtlsudp_ctor(void)
     };
     int i, num_prefixes = sizeof(prefixes) / sizeof(char *);
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
-    char indexname6[] = "_netsnmp_addr_info6";
+    static const char indexname6[] = "_netsnmp_addr_info6";
 #endif
 
     DEBUGMSGTL(("dtlsudp", "registering DTLS constructor\n"));
@@ -1571,13 +1637,13 @@ netsnmp_dtlsudp_ctor(void)
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
     if (!openssl_addr_index6)
         openssl_addr_index6 =
-            SSL_get_ex_new_index(0, indexname6, NULL, NULL, NULL);
+            SSL_get_ex_new_index(0, NETSNMP_REMOVE_CONST(void *, indexname6),
+                                 NULL, NULL, NULL);
 #endif
 
     dtlsudpDomain.name = netsnmpDTLSUDPDomain;
     dtlsudpDomain.name_length = netsnmpDTLSUDPDomain_len;
-    dtlsudpDomain.prefix = (const char**)calloc(num_prefixes + 1,
-                                                sizeof(char *));
+    dtlsudpDomain.prefix = calloc(num_prefixes + 1, sizeof(char *));
     for (i = 0; i < num_prefixes; ++ i)
         dtlsudpDomain.prefix[i] = prefixes[i];
 
@@ -1587,7 +1653,8 @@ netsnmp_dtlsudp_ctor(void)
 
     if (!openssl_addr_index)
         openssl_addr_index =
-            SSL_get_ex_new_index(0, indexname, NULL, NULL, NULL);
+            SSL_get_ex_new_index(0, NETSNMP_REMOVE_CONST(void *, indexname),
+                                 NULL, NULL, NULL);
 
     netsnmp_tdomain_register(&dtlsudpDomain);
 }
@@ -1600,21 +1667,13 @@ netsnmp_dtlsudp_ctor(void)
 int cookie_initialized=0;
 unsigned char cookie_secret[NETSNMP_COOKIE_SECRET_LENGTH];
 
-typedef union {
-       struct sockaddr sa;
-       struct sockaddr_in s4;
-#ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
-       struct sockaddr_in6 s6;
-#endif
-} _peer_union;
-
 int netsnmp_dtls_gen_cookie(SSL *ssl, unsigned char *cookie,
                             unsigned int *cookie_len)
 {
     unsigned char *buffer, result[EVP_MAX_MD_SIZE];
     unsigned int length, resultlength;
     bio_cache *cachep = NULL;
-    _peer_union *peer;
+    const netsnmp_sockaddr_storage *peer;
 
     /* Initialize a random secret */
     if (!cookie_initialized) {
@@ -1633,19 +1692,19 @@ int netsnmp_dtls_gen_cookie(SSL *ssl, unsigned char *cookie,
         snmp_log(LOG_ERR, "dtls: failed to get the peer address\n");
         return 0;
     }
-    peer = (_peer_union *)&cachep->sas;
+    peer = &cachep->sas;
 
     /* Create buffer with peer's address and port */
     length = 0;
     switch (peer->sa.sa_family) {
     case AF_INET:
         length += sizeof(struct in_addr);
-        length += sizeof(peer->s4.sin_port);
+        length += sizeof(peer->sin.sin_port);
         break;
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
     case AF_INET6:
         length += sizeof(struct in6_addr);
-        length += sizeof(peer->s6.sin6_port);
+        length += sizeof(peer->sin6.sin6_port);
         break;
 #endif
     default:
@@ -1662,19 +1721,19 @@ int netsnmp_dtls_gen_cookie(SSL *ssl, unsigned char *cookie,
     switch (peer->sa.sa_family) {
     case AF_INET:
         memcpy(buffer,
-               &peer->s4.sin_port,
-               sizeof(peer->s4.sin_port));
-        memcpy(buffer + sizeof(peer->s4.sin_port),
-               &peer->s4.sin_addr,
+               &peer->sin.sin_port,
+               sizeof(peer->sin.sin_port));
+        memcpy(buffer + sizeof(peer->sin.sin_port),
+               &peer->sin.sin_addr,
                sizeof(struct in_addr));
         break;
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
     case AF_INET6:
         memcpy(buffer,
-               &peer->s6.sin6_port,
-               sizeof(peer->s6.sin6_port));
-        memcpy(buffer + sizeof(peer->s6.sin6_port),
-               &peer->s6.sin6_addr,
+               &peer->sin6.sin6_port,
+               sizeof(peer->sin6.sin6_port));
+        memcpy(buffer + sizeof(peer->sin6.sin6_port),
+               &peer->sin6.sin6_addr,
                sizeof(struct in6_addr));
         break;
 #endif
@@ -1696,13 +1755,15 @@ int netsnmp_dtls_gen_cookie(SSL *ssl, unsigned char *cookie,
     return 1;
 }
 
-int netsnmp_dtls_verify_cookie(SSL *ssl, unsigned char *cookie,
+int netsnmp_dtls_verify_cookie(SSL *ssl,
+                               SECOND_APPVERIFY_COOKIE_CB_ARG_QUALIFIER
+                               unsigned char *cookie,
                                unsigned int cookie_len)
 {
     unsigned char *buffer, result[EVP_MAX_MD_SIZE];
     unsigned int length, resultlength, rc;
     bio_cache *cachep = NULL;
-    _peer_union *peer;
+    const netsnmp_sockaddr_storage *peer;
 
     /* If secret isn't initialized yet, the cookie can't be valid */
     if (!cookie_initialized)
@@ -1715,19 +1776,19 @@ int netsnmp_dtls_verify_cookie(SSL *ssl, unsigned char *cookie,
         snmp_log(LOG_ERR, "dtls: failed to get the peer address\n");
         return 0;
     }
-    peer = (_peer_union *)&cachep->sas;
+    peer = &cachep->sas;
 
     /* Create buffer with peer's address and port */
     length = 0;
     switch (peer->sa.sa_family) {
     case AF_INET:
         length += sizeof(struct in_addr);
-        length += sizeof(peer->s4.sin_port);
+        length += sizeof(peer->sin.sin_port);
         break;
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
     case AF_INET6:
         length += sizeof(struct in6_addr);
-        length += sizeof(peer->s6.sin6_port);
+        length += sizeof(peer->sin6.sin6_port);
         break;
 #endif
     default:
@@ -1745,19 +1806,19 @@ int netsnmp_dtls_verify_cookie(SSL *ssl, unsigned char *cookie,
     switch (peer->sa.sa_family) {
     case AF_INET:
         memcpy(buffer,
-               &peer->s4.sin_port,
-               sizeof(peer->s4.sin_port));
-        memcpy(buffer + sizeof(peer->s4.sin_port),
-               &peer->s4.sin_addr,
+               &peer->sin.sin_port,
+               sizeof(peer->sin.sin_port));
+        memcpy(buffer + sizeof(peer->sin.sin_port),
+               &peer->sin.sin_addr,
                sizeof(struct in_addr));
         break;
 #ifdef NETSNMP_TRANSPORT_UDPIPV6_DOMAIN
     case AF_INET6:
         memcpy(buffer,
-               &peer->s6.sin6_port,
-               sizeof(peer->s6.sin6_port));
-        memcpy(buffer + sizeof(peer->s6.sin6_port),
-               &peer->s6.sin6_addr,
+               &peer->sin6.sin6_port,
+               sizeof(peer->sin6.sin6_port));
+        memcpy(buffer + sizeof(peer->sin6.sin6_port),
+               &peer->sin6.sin6_addr,
                sizeof(struct in6_addr));
         break;
 #endif
