@@ -7,6 +7,11 @@
  * Copyright Copyright 2003 Sun Microsystems, Inc. All rights reserved.
  * Use is subject to license terms specified in the COPYING file
  * distributed with the Net-SNMP package.
+ *
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
  */
 
 #include <net-snmp/net-snmp-config.h>
@@ -80,11 +85,6 @@ static netsnmp_tdomain udpDomain;
  */
 typedef netsnmp_indexed_addr_pair netsnmp_udp_addr_pair;
 
-/*
- * not static, since snmpUDPIPv6Domain needs it, but not public, either.
- * (ie don't put it in a public header.)
- */
-void _netsnmp_udp_sockopt_set(int fd, int server);
 int
 netsnmp_sockaddr_in2(struct sockaddr_in *addr,
                      const char *inpeername, const char *default_target);
@@ -95,41 +95,36 @@ netsnmp_sockaddr_in2(struct sockaddr_in *addr,
  */
 
 char *
-netsnmp_udp_fmtaddr(netsnmp_transport *t, void *data, int len)
+netsnmp_udp_fmtaddr(netsnmp_transport *t, const void *data, int len)
 {
     return netsnmp_ipv4_fmtaddr("UDP", t, data, len);
 }
 
 
-#if defined(HAVE_IP_PKTINFO) || defined(HAVE_IP_RECVDSTADDR)
+#if defined(HAVE_IP_PKTINFO) || (defined(HAVE_IP_RECVDSTADDR) && defined(HAVE_IP_SENDSRCADDR))
 
 int netsnmp_udp_recvfrom(int s, void *buf, int len, struct sockaddr *from, socklen_t *fromlen, struct sockaddr *dstip, socklen_t *dstlen, int *if_index)
 {
-    /** udpiv4 just calls udpbase. should we skip directly to there? */
+    /** udpipv4 just calls udpbase. should we skip directly to there? */
     return netsnmp_udpipv4_recvfrom(s, buf, len, from, fromlen, dstip, dstlen,
                                     if_index);
 }
 
-int netsnmp_udp_sendto(int fd, struct in_addr *srcip, int if_index, struct sockaddr *remote,
-                void *data, int len)
+int netsnmp_udp_sendto(int fd, const struct in_addr *srcip, int if_index,
+                       const struct sockaddr *remote, const void *data, int len)
 {
-    /** udpiv4 just calls udpbase. should we skip directly to there? */
+    /** udpipv4 just calls udpbase. should we skip directly to there? */
     return netsnmp_udpipv4_sendto(fd, srcip, if_index, remote, data, len);
 }
 #endif /* HAVE_IP_PKTINFO || HAVE_IP_RECVDSTADDR */
 
 /*
- * Open a UDP-based transport for SNMP.  Local is TRUE if addr is the local
- * address to bind to (i.e. this is a server-type session); otherwise addr is 
- * the remote address to send things to.  
+ * Common initialization of udp transport.
  */
 
-netsnmp_transport *
-netsnmp_udp_transport(struct sockaddr_in *addr, int local)
+static netsnmp_transport *
+netsnmp_udp_transport_base(netsnmp_transport *t)
 {
-    netsnmp_transport *t = NULL;
-
-    t = netsnmp_udpipv4base_transport(addr, local);
     if (NULL == t) {
         return NULL;
     }
@@ -151,10 +146,47 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
     t->f_close    = netsnmp_socketbase_close;
     t->f_accept   = NULL;
     t->f_fmtaddr  = netsnmp_udp_fmtaddr;
+    t->f_get_taddr = netsnmp_ipv4_get_taddr;
 
     return t;
 }
 
+/*
+ * Open a UDP-based transport for SNMP.  Local is TRUE if addr is the local
+ * address to bind to (i.e. this is a server-type session); otherwise addr is
+ * the remote address to send things to.
+ */
+netsnmp_transport *
+netsnmp_udp_transport(const struct sockaddr_in *addr, int local)
+{
+    netsnmp_transport *t = NULL;
+
+    t = netsnmp_udpipv4base_transport(addr, local);
+    if (NULL != t) {
+        netsnmp_udp_transport_base(t);
+    }
+    return t;
+}
+
+/*
+ * Open a UDP-based transport for SNMP.  Local is TRUE if addr is the local
+ * address to bind to (i.e. this is a server-type session); otherwise addr is
+ * the remote address to send things to and src_addr is the optional addr
+ * to send from.
+ */
+netsnmp_transport *
+netsnmp_udp_transport_with_source(const struct sockaddr_in *addr, int local,
+                                  const struct sockaddr_in *src_addr)
+
+{
+    netsnmp_transport *t = NULL;
+
+    t = netsnmp_udpipv4base_transport_with_source(addr, local, src_addr);
+    if (NULL != t) {
+        netsnmp_udp_transport_base(t);
+    }
+    return t;
+}
 #if !defined(NETSNMP_DISABLE_SNMPV1) || !defined(NETSNMP_DISABLE_SNMPV2C)
 /*
  * The following functions provide the "com2sec" configuration token
@@ -164,28 +196,119 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
 #define EXAMPLE_NETWORK		"NETWORK"
 #define EXAMPLE_COMMUNITY	"COMMUNITY"
 
-typedef struct com2SecEntry_s {
+struct com2SecEntry_s {
     const char *secName;
     const char *contextName;
     struct com2SecEntry_s *next;
     in_addr_t   network;
     in_addr_t   mask;
+    int         negate;
     const char  community[1];
-} com2SecEntry;
+};
 
 static com2SecEntry   *com2SecList = NULL, *com2SecListLast = NULL;
 
+int
+netsnmp_udp_com2SecEntry_create(com2SecEntry **entryp, const char *community,
+                    const char *secName, const char *contextName,
+                    struct in_addr *network, struct in_addr *mask,
+                    int negate)
+{
+    int communityLen, secNameLen, contextNameLen, len;
+    com2SecEntry* e;
+    char* last;
+    struct in_addr dflt_network, dflt_mask;
+
+    if (NULL != entryp)
+        *entryp = NULL;
+
+    if (NULL == community || NULL == secName)
+        return C2SE_ERR_MISSING_ARG;
+
+    if (NULL == network) {
+        network = &dflt_network;
+        dflt_network.s_addr = 0;
+    }
+    if (NULL == mask) {
+        mask = &dflt_mask;
+        dflt_mask.s_addr = 0;
+    }
+
+    /** Check that the network and mask are consistent. */
+    if (network->s_addr & ~mask->s_addr)
+        return C2SE_ERR_MASK_MISMATCH;
+
+    communityLen = strlen(community);
+    if (communityLen > COMMUNITY_MAX_LEN)
+        return C2SE_ERR_COMMUNITY_TOO_LONG;
+
+    secNameLen = strlen(secName);
+    if (secNameLen > VACM_MAX_STRING)
+        return C2SE_ERR_SECNAME_TOO_LONG;
+
+    contextNameLen = contextName ? strlen(contextName) : 0;
+    if (contextNameLen > VACM_MAX_STRING)
+        return C2SE_ERR_CONTEXT_TOO_LONG;
+
+    /** alloc space for struct + 3 strings with NULLs */
+    len = offsetof(com2SecEntry, community) + communityLen + secNameLen +
+        contextNameLen + 3;
+    e = (com2SecEntry*)calloc(len, 1);
+    if (e == NULL)
+        return C2SE_ERR_MEMORY;
+    last = ((char*)e) + offsetof(com2SecEntry, community);
+
+    DEBUGIF("netsnmp_udp_parse_security") {
+        char buf1[INET_ADDRSTRLEN];
+        char buf2[INET_ADDRSTRLEN];
+        DEBUGMSGTL(("netsnmp_udp_parse_security",
+                    "<\"%s\", %s/%s> => \"%s\"\n", community,
+                    inet_ntop(AF_INET, network, buf1, sizeof(buf1)),
+                    inet_ntop(AF_INET, mask, buf2, sizeof(buf2)),
+                    secName));
+    }
+
+    memcpy(last, community, communityLen);
+    last += communityLen + 1;
+    memcpy(last, secName, secNameLen);
+    e->secName = last;
+    last += secNameLen + 1;
+    if (contextNameLen) {
+        memcpy(last, contextName, contextNameLen);
+        e->contextName = last;
+    } else
+        e->contextName = last - 1;
+    e->network = network->s_addr;
+    e->mask = mask->s_addr;
+    e->negate = negate;
+    e->next = NULL;
+
+    if (com2SecListLast != NULL) {
+        com2SecListLast->next = e;
+        com2SecListLast = e;
+    } else {
+        com2SecListLast = com2SecList = e;
+    }
+
+    if (NULL != entryp)
+        *entryp = e;
+
+    return C2SE_ERR_SUCCESS;
+}
+
 void
 netsnmp_udp_parse_security(const char *token, char *param)
 {
-    char            secName[VACMSTRINGLEN + 1];
-    size_t          secNameLen;
-    char            contextName[VACMSTRINGLEN + 1];
-    size_t          contextNameLen;
-    char            community[COMMUNITY_MAX_LEN + 1];
-    size_t          communityLen;
-    char            source[270]; /* dns-name(253)+/(1)+mask(15)+\0(1) */
+    /** copy_nword does null term, so we need vars of max size + 2. */
+    /** (one for null, one to detect param too long */
+    char            secName[VACMSTRINGLEN]; /* == VACM_MAX_STRING + 2 */
+    char            contextName[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN + 2];
+    char            source[271]; /* !(1)+dns-name(253)+/(1)+mask(15)+\0(1) */
+    char            *sourcep;
     struct in_addr  network, mask;
+    int             negate;
+    int rc;
 
     /*
      * Get security, source address/netmask and community strings.
@@ -198,27 +321,17 @@ netsnmp_udp_parse_security(const char *token, char *param)
             return;
         }
         param = copy_nword( param, contextName, sizeof(contextName));
-        contextNameLen = strlen(contextName) + 1;
-        if (contextNameLen > VACMSTRINGLEN) {
-            config_perror("context name too long");
-            return;
-        }
         if (!param) {
             config_perror("missing NAME parameter");
             return;
         }
         param = copy_nword( param, secName, sizeof(secName));
-    } else {
-        contextNameLen = 0;
-    }
+    } else
+        contextName[0] = '\0';
 
-    secNameLen = strlen(secName) + 1;
-    if (secNameLen == 1) {
+    if (secName[0] == '\0') {
         config_perror("empty NAME parameter");
         return;
-    } else if (secNameLen > VACMSTRINGLEN) {
-        config_perror("security name too long");
-        return;
     }
 
     if (!param) {
@@ -244,12 +357,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
         config_perror("empty COMMUNITY parameter");
         return;
     }
-    communityLen = strlen(community) + 1;
-    if (communityLen >= COMMUNITY_MAX_LEN) {
-        config_perror("community name too long");
-        return;
-    }
-    if (communityLen == sizeof(EXAMPLE_COMMUNITY) &&
+    if ((strlen(community) + 1) == sizeof(EXAMPLE_COMMUNITY) &&
         memcmp(community, EXAMPLE_COMMUNITY, sizeof(EXAMPLE_COMMUNITY)) == 0) {
         config_perror("example config COMMUNITY not properly configured");
         return;
@@ -259,17 +367,26 @@ netsnmp_udp_parse_security(const char *token, char *param)
     if (strcmp(source, "default") == 0) {
         network.s_addr = 0;
         mask.s_addr = 0;
+        negate = 0;
     } else {
+        if (*source == '!') {
+            negate = 1;
+            sourcep = source + 1;
+        } else {
+            negate = 0;
+            sourcep = source;
+        }
+
         /* Split the source/netmask parts */
-        char *strmask = strchr(source, '/');
+        char *strmask = strchr(sourcep, '/');
         if (strmask != NULL)
             /* Mask given. */
             *strmask++ = '\0';
 
         /* Try interpreting as a dotted quad. */
-        if (inet_pton(AF_INET, source, &network) == 0) {
+        if (inet_pton(AF_INET, sourcep, &network) == 0) {
             /* Nope, wasn't a dotted quad.  Must be a hostname. */
-            int ret = netsnmp_gethostbyname_v4(source, &network.s_addr);
+            int ret = netsnmp_gethostbyname_v4(sourcep, &network.s_addr);
             if (ret < 0) {
                 config_perror("cannot resolve source hostname");
                 return;
@@ -287,7 +404,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
             if (*cp == '\0') {
                 if (0 < maskLen && maskLen <= 32)
                     mask.s_addr = htonl((in_addr_t)(~0UL << (32 - maskLen)));
-                else if (maskLen == 0)
+                else if (0 == maskLen)
                     mask.s_addr = 0;
                 else {
                     config_perror("bad mask length");
@@ -308,56 +425,62 @@ netsnmp_udp_parse_security(const char *token, char *param)
         }
     }
 
-    {
-        void* v = malloc(offsetof(com2SecEntry, community) + communityLen +
-                         secNameLen + contextNameLen);
-
-        com2SecEntry* e = (com2SecEntry*)v;
-        char* last = ((char*)v) + offsetof(com2SecEntry, community);
-
-        if (v == NULL) {
-            config_perror("memory error");
-            return;
-        }
-
-        /*
-         * Everything is okay.  Copy the parameters to the structure allocated
-         * above and add it to END of the list.
-         */
-
-        {
-          char buf1[INET_ADDRSTRLEN];
-          char buf2[INET_ADDRSTRLEN];
-          DEBUGMSGTL(("netsnmp_udp_parse_security",
-                      "<\"%s\", %s/%s> => \"%s\"\n", community,
-                      inet_ntop(AF_INET, &network, buf1, sizeof(buf1)),
-                      inet_ntop(AF_INET, &mask, buf2, sizeof(buf2)),
-                      secName));
-        }
-
-        memcpy(last, community, communityLen);
-        last += communityLen;
-        memcpy(last, secName, secNameLen);
-        e->secName = last;
-        last += secNameLen;
-        if (contextNameLen) {
-            memcpy(last, contextName, contextNameLen);
-            e->contextName = last;
-        } else
-            e->contextName = last - 1;
-        e->network = network.s_addr;
-        e->mask = mask.s_addr;
-        e->next = NULL;
-
-        if (com2SecListLast != NULL) {
-            com2SecListLast->next = e;
-            com2SecListLast = e;
-        } else {
-            com2SecListLast = com2SecList = e;
-        }
+    /*
+     * Everything is okay.  Copy the parameters to the structure allocated
+     * above and add it to END of the list.
+     */
+    rc = netsnmp_udp_com2SecEntry_create(NULL, community, secName, contextName,
+                                         &network, &mask, negate);
+    switch(rc) {
+        case C2SE_ERR_SUCCESS:
+            break;
+        case C2SE_ERR_CONTEXT_TOO_LONG:
+            config_perror("context name too long");
+            break;
+        case C2SE_ERR_COMMUNITY_TOO_LONG:
+            config_perror("community name too long");
+            break;
+        case C2SE_ERR_SECNAME_TOO_LONG:
+            config_perror("security name too long");
+            break;
+        case C2SE_ERR_MASK_MISMATCH:
+            config_perror("source/mask mismatch");
+            break;
+        case C2SE_ERR_MISSING_ARG:
+        default:
+            config_perror("unexpected error; could not create com2SecEntry");
     }
 }
 
+void
+netsnmp_udp_com2Sec_free(com2SecEntry *e)
+{
+    free(e);
+}
+
+int
+netsnmp_udp_com2SecList_remove(com2SecEntry *e)
+{
+    com2SecEntry   *c = com2SecList, *p = NULL;
+    for (; c != NULL; p = c, c = c->next) {
+        if (e == c)
+            break;
+    }
+    if (NULL == c)
+        return 1;
+
+    if (NULL == p)
+        com2SecList = e->next;
+    else
+        p->next = e->next;
+    e->next = NULL;
+
+    if (e == com2SecListLast)
+        com2SecListLast = p;
+
+    return 0;
+}
+
 void
 netsnmp_udp_com2SecList_free(void)
 {
@@ -365,7 +488,7 @@ netsnmp_udp_com2SecList_free(void)
     while (e != NULL) {
         com2SecEntry   *tmp = e;
         e = e->next;
-        free(tmp);
+        netsnmp_udp_com2Sec_free(tmp);
     }
     com2SecList = com2SecListLast = NULL;
 }
@@ -455,6 +578,14 @@ netsnmp_udp_getSecName(void *opaque, int olength,
 	    (memcmp(community, c->community, community_len) == 0) &&
             ((from->sin_addr.s_addr & c->mask) == c->network)) {
             DEBUGMSG(("netsnmp_udp_getSecName", "... SUCCESS\n"));
+            if (c->negate) {
+                /*
+                 * If we matched a negative entry, then we are done - claim that we
+                 * matched nothing.
+                 */
+                DEBUGMSG(("netsnmp_udp_getSecName", "... <negative entry>\n"));
+                break;
+            }
             if (secName != NULL) {
                 *secName = c->secName;
                 *contextName = c->contextName;
@@ -484,19 +615,24 @@ netsnmp_udp_create_tstring(const char *str, int local,
     }
 }
 
+netsnmp_transport *
+netsnmp_udp_create_tspec(netsnmp_tdomain_spec *tspec)
+{
+    netsnmp_transport *t = netsnmp_udpipv4base_tspec_transport(tspec);
+    if (NULL != t) {
+        netsnmp_udp_transport_base(t);
+    }
+    return t;
+
+}
 
 netsnmp_transport *
-netsnmp_udp_create_ostring(const u_char * o, size_t o_len, int local)
+netsnmp_udp_create_ostring(const void *o, size_t o_len, int local)
 {
-    struct sockaddr_in addr;
+    struct sockaddr_in sin;
 
-    if (o_len == 6) {
-        unsigned short porttmp = (o[4] << 8) + o[5];
-        addr.sin_family = AF_INET;
-        memcpy((u_char *) & (addr.sin_addr.s_addr), o, 4);
-        addr.sin_port = htons(porttmp);
-        return netsnmp_udp_transport(&addr, local);
-    }
+    if (netsnmp_ipv4_ostring_to_sockaddr(&sin, o, o_len))
+        return netsnmp_udp_transport(&sin, local);
     return NULL;
 }
 
@@ -511,6 +647,7 @@ netsnmp_udp_ctor(void)
 
     udpDomain.f_create_from_tstring     = NULL;
     udpDomain.f_create_from_tstring_new = netsnmp_udp_create_tstring;
+    udpDomain.f_create_from_tspec       = netsnmp_udp_create_tspec;
     udpDomain.f_create_from_ostring     = netsnmp_udp_create_ostring;
 
     netsnmp_tdomain_register(&udpDomain);
