@@ -14,21 +14,21 @@ use constant true => 1;
 my $target_arch = $ENV{TARGET_CPU} ? $ENV{TARGET_CPU} : $ENV{Platform} ?
                   $ENV{Platform} : "x86";
 $target_arch = lc $target_arch;
-if ($target_arch ne "x86" && $target_arch ne "x64" && $target_arch ne "arm64") {
+if ($target_arch ne "x86" && $target_arch ne "x64") {
     print "Error: unsupported target architecture $target_arch\n";
     die;
 }
 my @perl_arch = split(/-/, $Config{archname});
 my $openssl = false;
-my $default_openssldir = $target_arch eq "x64" || $target_arch eq "arm64" ?
+my $default_openssldir = $target_arch eq "x64" ?
     "C:\\OpenSSL-Win64" : "C:\\OpenSSL-Win32";
 my $default_opensslincdir = $default_openssldir . "\\include";
 my $opensslincdir = $default_opensslincdir;
-my $default_openssllibdir = $default_openssldir . "\\lib\\VC\\x64";
+my $default_openssllibdir = $default_openssldir . "\\lib\\VC";
 my $openssllibdir = $default_openssllibdir;
 my $b_ipv6 = false;
 my $b_winextdll = false;
-my $sdk = true;
+my $sdk = false;
 my $default_install_base = "c:/usr";
 my $install_base = $default_install_base;
 my $install = true;
@@ -58,22 +58,23 @@ while (1) {
   print "1.  OpenSSL support:                " . ($openssl ? "enabled" : "disabled"). "\n";
   print "2.  OpenSSL include directory:      " . $opensslincdir. "\n";
   print "3.  OpenSSL library director:       " . $openssllibdir. "\n";
+  print "4.  Platform SDK support:           " . ($sdk ? "enabled" : "disabled") . "\n";
   print "\n";
-  print "4.  Install path:                   " . $install_base . "\n";
-  print "5.  Install after build:            " . ($install ? "enabled" : "disabled") . "\n";
+  print "5.  Install path:                   " . $install_base . "\n";
+  print "6.  Install after build:            " . ($install ? "enabled" : "disabled") . "\n";
   print "\n";
-  print "6.  Perl modules:                   " . ($perl ? "enabled" : "disabled") . "\n";
-  print "7.  Install perl modules:           " . ($perl_install ? "enabled" : "disabled") . "\n";
+  print "7.  Perl modules:                   " . ($perl ? "enabled" : "disabled") . "\n";
+  print "8.  Install perl modules:           " . ($perl_install ? "enabled" : "disabled") . "\n";
   print "\n";
-  print "8.  Quiet build (logged):           " . ($logging ? "enabled" : "disabled") . "\n";
-  print "9.  Debug mode:                     " . ($debug ? "enabled" : "disabled") . "\n";
+  print "9.  Quiet build (logged):           " . ($logging ? "enabled" : "disabled") . "\n";
+  print "10. Debug mode:                     " . ($debug ? "enabled" : "disabled") . "\n";
   print "\n";
-  print "10. IPv6 transports:                " . ($b_ipv6 ? "enabled" : "disabled") . "\n";
-  print "11. winExtDLL agent:                " . ($b_winextdll ? "enabled" : "disabled") . "\n";
+  print "11. IPv6 transports (requires SDK): " . ($b_ipv6 ? "enabled" : "disabled") . "\n";
+  print "12. winExtDLL agent (requires SDK): " . ($b_winextdll ? "enabled" : "disabled") . "\n";
   print "\n";
-  print "12. Link type:                      " . ($link_dynamic ? "dynamic" : "static") . "\n";
+  print "13. Link type:                      " . ($link_dynamic ? "dynamic" : "static") . "\n";
   print "\n";
-  print "13. Install development files       " . ($install_devel ? "enabled" : "disabled") . "\n";
+  print "14. Install development files       " . ($install_devel ? "enabled" : "disabled") . "\n";
   print "\nF.  Finished - start build\n";
   print "Q.  Quit - abort build\n\n";
   print "Select option to set / toggle: ";
@@ -94,45 +95,48 @@ while (1) {
     $openssllibdir =~ s/\\/\//g;
     $openssllibdir = $default_openssllibdir if ($openssllibdir eq "");
   }
-  elsif ($option eq "10") {
+  elsif ($option eq "4") {
+    $sdk = !$sdk;
+  }
+  elsif ($option eq "11") {
     $b_ipv6 = !$b_ipv6;
     if ($b_ipv6 && !$sdk) {
       print "\n\n* SDK required for IPv6 and has been automatically enabled";
       $sdk = true;
     }
   }
-  elsif ($option eq "11") {
+  elsif ($option eq "12") {
     $b_winextdll = !$b_winextdll;
     if ($b_winextdll && !$sdk) {
       print "\n\n* SDK required for IPv6 and has been automatically enabled";
       $sdk = true;
     }
   }
-  elsif ($option eq "4") {
+  elsif ($option eq "5") {
     print "Please enter the new install path [$default_install_base]: ";
     chomp ($install_base = <>);
     $install_base =~ s/\\/\//g;
     $install_base = $default_install_base if ($install_base eq "");
   }
-  elsif ($option eq "5") {
+  elsif ($option eq "6") {
     $install = !$install;
   }
-  elsif ($option eq "13") {
+  elsif ($option eq "14") {
     $install_devel = !$install_devel;
   }
-  elsif ($option eq "6") {
+  elsif ($option eq "7") {
     $perl = !$perl;
   }
-  elsif ($option eq "7") {
+  elsif ($option eq "8") {
     $perl_install = !$perl_install;
   }
-  elsif ($option eq "8") {
+  elsif ($option eq "9") {
     $logging = !$logging;
   }
-  elsif ($option eq "9") {
+  elsif ($option eq "10") {
     $debug = !$debug;
   }
-  elsif ($option eq "12") {
+  elsif ($option eq "13") {
     $link_dynamic = !$link_dynamic;
   }
   elsif (lc($option) eq "f") {
@@ -148,15 +152,10 @@ if ($perl && $perl_arch[1] ne $target_arch) {
     die;
 }
 
-my $openssllibsubdir = $link_dynamic ? "\\MD" : "\\MT";
-if ($debug) {
-  $openssllibsubdir .= "d";
-}
-
 my $linktype = $link_dynamic ? "dynamic" : "static";
-$configOpts = (($openssl ? "--with-ssl --enable-blumenthal-aes" : "")	 . " " .
+$configOpts = (($openssl ? "--with-ssl" : "")		. " " .
                ($opensslincdir ? "--with-sslincdir=$opensslincdir" : "") . " " .
-               ($openssllibdir ? "--with-ssllibdir=$openssllibdir" . "$openssllibsubdir" : "") . " " .
+               ($openssllibdir ? "--with-ssllibdir=$openssllibdir" : "") . " " .
                ($sdk ? "--with-sdk" : "")		. " " .
                ($b_ipv6 ? "--with-ipv6" : "")		. " " .
                ($b_winextdll ? "--with-winextdll" : "") . " " .
